package com.ims.eval.cache;

import com.ims.eval.entity.BinSection;
import com.ims.eval.entity.BinStage;
import com.ims.eval.entity.DataDictionary;
import com.ims.eval.entity.IndicatorType;
import com.ims.eval.service.IBinSectionService;
import com.ims.eval.service.IBinStageService;
import com.ims.eval.service.IDataDictionaryService;
import com.ims.eval.service.IIndicatorTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 初始化缓存类
 */
@Component
public class CacheContext implements CommandLineRunner {

	@Autowired
	private IDataDictionaryService dataDictionaryService;

	@Autowired
	private IIndicatorTypeService iIndicatorTypeService;

	@Autowired
	private IBinSectionService binSectionService;

	@Autowired
	private IBinStageService binStageService;


	//初始化字典表
	public static List<DataDictionary> ddList = new ArrayList<>();
	public static Map<String, DataDictionary> ddMap = new HashMap<>();
	public static Map<String, String> ddNameMap = new HashMap<>();
	public static Map<String, String> ddValueMap = new HashMap<>();
	public static Map<String, List<DataDictionary>> ddSuperKeyMap  = new HashMap<>();


	//指标分类
	public static List<IndicatorType> itList = new ArrayList<>();

	//经营阶段分类
	public static List<BinStage> bseList = new ArrayList<>();
	public static Map<String, String> bseIdMap = new HashMap<>();
	public static Map<String, BinStage> bseIdObject = new HashMap<>();
	public static Map<String, BinStage> bseCodeObject = new HashMap<>();



	//生产经营业务分类
	public static List<BinSection> bsnList = new ArrayList<>();
	public static Map<String, BinSection> bsnIdObject = new HashMap<>();
	public static Map<String, BinSection> bsnCodeObject = new HashMap<>();
	public static Map<String, String> bsnIdName = new HashMap<>();



	@Override
	public void run(String... args) throws Exception {
		initDataDictionary();
		initIndicatorType();
		initBinStage();
		initBinSection();

	}


	/**
	 * 初始化数据字典表
	 */
	public void initDataDictionary(){
		//清理集合中的数据
		ddList.clear();
		ddMap.clear();
		ddNameMap.clear();
		ddValueMap.clear();
		ddSuperKeyMap.clear();
		//重新加载集合数据
		ddList = dataDictionaryService.list();
		ddList.stream().forEach(d -> {
			ddMap.put(d.getDataKey(), d);
			ddNameMap.put(d.getDataKey(), d.getKeyName());
			ddValueMap.put(d.getDataKey(), d.getKeyValue());
		});
		ddSuperKeyMap = ddList.stream().collect(Collectors.groupingBy(DataDictionary::getSuperKey));
	}


	/**
	 * 初始化指标分类
	 */
	public void initIndicatorType(){
		//清理集合中的数据
		itList.clear();

		itList = iIndicatorTypeService.list().stream().filter(t->!t.getDelFlag()).collect(Collectors.toList());
		itList.sort(Comparator.comparing(IndicatorType::getOrderNum));
	}



	/**
	 * 初始化经营阶段分类
	 */
	public void initBinStage(){
		//清理集合中的数据
		bseList.clear();
		bseIdMap.clear();
		bseIdObject.clear();
		bseCodeObject.clear();

		bseList = binStageService.list().stream().filter(t->!t.getDelFlag()).collect(Collectors.toList());
		bseList.sort(Comparator.comparing(BinStage::getOrderNum));
		bseList.stream().forEach(d -> {
			bseIdMap.put(d.getId(), d.getStageName());
			bseIdObject.put(d.getId(), d);
			bseCodeObject.put(d.getStageCode(), d);
		});


	}

	/**
	 * 初始化生产经营业务分类
	 */
	public void initBinSection(){
		//清理集合中的数据
		bsnList.clear();
		bsnIdObject.clear();
		bsnCodeObject.clear();


		bsnList = binSectionService.list().stream().filter(t->!t.getDelFlag()).collect(Collectors.toList());
		bsnList.sort(Comparator.comparing(BinSection::getOrderNum));
		bsnList.stream().forEach(d -> {
			bsnIdObject.put(d.getId(), d);
		});

		bsnList.stream().forEach(d -> {
			bsnCodeObject.put(d.getSectionCode(), d);
			bsnIdName.put(d.getId(), d.getSectionName());

		});
	}




}
