package com.ims.eval.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationDept;
import com.ims.eval.entity.EvaluationDeptBusinessContent;
import com.ims.eval.entity.EvaluationDeptBusinessPlan;
import com.ims.eval.entity.custom.Path;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationDeptBusinessContentService;
import com.ims.eval.service.IEvaluationDeptBusinessPlanService;
import com.ims.eval.util.ExcelUtil;
import com.ims.eval.util.WordUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.*;

/**
 * 本部部门业绩指标计划
 *
 * @author hlf
 * @date 2023/4/26 14:15
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-plan")
public class DepartmentalPerformanceIndicatorPlanController {

	@Autowired
	private IEvaluationDeptBusinessPlanService evaluationDeptBusinessPlanService;

	@Autowired
	private IEvaluationDeptBusinessContentService evaluationDeptBusinessContentService;

	/**
	 * 部门考评业务列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param responsibilityCode 业务编码
	 * @param des                业务简述
	 * @param appraisalYear      月度
	 * @param stage              流程状态
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
		@RequestParam(value = "des", required = false) String des,
		@RequestParam(value = "appraisalYear", required = false) String appraisalYear,
		@RequestParam(value = "stage", required = false) String stage,
		@RequestParam(value = "deptId", required = false) String deptId) {
		IPage<EvaluationDeptBusinessPlan> list = evaluationDeptBusinessPlanService.listPage(pageNum, pageSize, responsibilityCode, des, appraisalYear, stage, deptId);
		return R.ok().data(list);
	}

	/**
	 * 部门考评业务信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationDeptBusinessPlan> list = evaluationDeptBusinessPlanService.list();
		return R.ok().data(list);
	}

	/**
	 * 新增部门考评业务信息
	 *
	 * @param evaluationDeptBusinessPlan 部门考评业务实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationDeptBusinessPlan evaluationDeptBusinessPlan) {
		try {
			QueryWrapper<EvaluationDeptBusinessPlan> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(evaluationDeptBusinessPlan.getAppraisalYear())) {
				qw.lambda().eq(EvaluationDeptBusinessPlan::getAppraisalYear, evaluationDeptBusinessPlan.getAppraisalYear());
			}
			EvaluationDeptBusinessPlan businessPlan = evaluationDeptBusinessPlanService.getOne(qw);
			if (ObjectUtil.isNotNull(businessPlan)) {
				return R.ok().message(evaluationDeptBusinessPlan.getAppraisalYear() + "年度业绩指标计划已存在！");
			}
			boolean b = evaluationDeptBusinessPlanService.save(evaluationDeptBusinessPlan);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 修改部门考评业务信息
	 *
	 * @param evaluationDeptBusinessPlan 部门考评业务实体
	 * @return 结果
	 */
	@PostMapping(value = "/modify")
	public R modify(@RequestBody EvaluationDeptBusinessPlan evaluationDeptBusinessPlan) {
		try {
			boolean b = evaluationDeptBusinessPlanService.updateById(evaluationDeptBusinessPlan);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("修改失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除部门考评业务信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = evaluationDeptBusinessPlanService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

	/**
	 * 详情-头部信息
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/detailsHead/{id}")
	public R detailsHead(@PathVariable("id") String id) {
		EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = evaluationDeptBusinessPlanService.getById(id);
		return R.ok().data(evaluationDeptBusinessPlan);
	}

	/**
	 * 详情
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/details/{id}/{deptId}")
	public R details(@PathVariable("id") String id, @PathVariable("deptId") String deptId) {
		List<EvaluationDeptBusinessContent> list = evaluationDeptBusinessContentService.detailsList(id, deptId);
		return R.ok().data(list);
	}

	/**
	 * 详情-修改/新增
	 *
	 * @param evaluationDeptBusinessContentList 修改内容
	 * @return 结果
	 */
	@PostMapping(value = "/update")
	public R update(@RequestBody List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList) {
		try {
			boolean b = evaluationDeptBusinessContentService.saveOrUpdateBatch(evaluationDeptBusinessContentList);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("修改失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 导入
	 *
	 * @param file 文件
	 * @return 结果
	 */
	@PostMapping(value = "/import")
	public R importData(@RequestParam("file") MultipartFile file) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					EvaluationDeptBusinessContent evaluationDeptBusinessContent = new EvaluationDeptBusinessContent();
					evaluationDeptBusinessContent.setId(str[0]);
					evaluationDeptBusinessContent.setBusinessPlanId(str[1]);
					evaluationDeptBusinessContent.setDeptId(str[2]);
					evaluationDeptBusinessContent.setDeptName(str[3]);
					evaluationDeptBusinessContent.setDeptCode(str[4]);
					evaluationDeptBusinessContent.setSerialNumber(str[5]);
					evaluationDeptBusinessContent.setTargetName(str[6]);
					evaluationDeptBusinessContent.setTargetValue(str[7]);
					evaluationDeptBusinessContent.setBaseScore(str[8]);
					evaluationDeptBusinessContent.setEvaluationCriteria(str[9]);
					evaluationDeptBusinessContent.setAuditStatus("-1");
					if (null == str[0] || "".equals(str[0])) {
						evaluationDeptBusinessContent.setCreateTime(DateUtil.date());
					} else {
						evaluationDeptBusinessContent.setUpdateTime(DateUtil.date());
					}
					evaluationDeptBusinessContentList.add(evaluationDeptBusinessContent);
				}
				boolean b = evaluationDeptBusinessContentService.saveOrUpdateBatch(evaluationDeptBusinessContentList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error().data("保存失败！");
				}
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.customError("上传文件为空！");
	}

	/**
	 * 修改状态
	 *
	 * @param id    主键
	 * @param stage 状态
	 * @return 结果
	 */
	@PostMapping(value = "/targetStart")
	public R targetStart(@RequestParam(value = "id") String id,
						 @RequestParam(value = "stage") String stage) {
		try {
			EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = evaluationDeptBusinessPlanService.getById(id);
			evaluationDeptBusinessPlan.setStage(stage);
			boolean b = evaluationDeptBusinessPlanService.updateById(evaluationDeptBusinessPlan);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 生成报表
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/generateReport/{id}")
	public R generateReport(@PathVariable("id") String id) throws Exception {
		Map<String, Object> dataMap = new HashMap<>();
		EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = evaluationDeptBusinessPlanService.getById(id);
		String[] deptIds = evaluationDeptBusinessPlan.getDeptId().split(",");
		String[] deptNames = evaluationDeptBusinessPlan.getDeptName().split(",");
		List<EvaluationDept> evaluationDeptList = new ArrayList<>();
		for (int i = 0; i < deptIds.length; i++) {
			EvaluationDept evaluationDept = new EvaluationDept();
			evaluationDept.setDeptId(deptIds[i]);
			evaluationDept.setDeptName(deptNames[i]);
			List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = evaluationDeptBusinessContentService.selectEvaluationDeptBusinessContentListByBusinessPlanId(id, deptIds[i]);
			if (evaluationDeptBusinessContentList.size() > 0) {
				evaluationDept.setEvaluationDeptBusinessContent(evaluationDeptBusinessContentList.get(0));
				if (evaluationDeptBusinessContentList.size() > 1) {
					evaluationDeptBusinessContentList.remove(0);
					evaluationDept.setList(evaluationDeptBusinessContentList);
				}
			}
			evaluationDeptList.add(evaluationDept);
		}
		dataMap.put("appraisalYear", evaluationDeptBusinessPlan.getAppraisalYear());
		dataMap.put("list", evaluationDeptList);
		String fileName = evaluationDeptBusinessPlan.getAppraisalYear() + "年度经营业绩指标计划.doc";
		WordUtil.generateWord(dataMap, "template_dbkp_zbjh.ftl", Path.getStatementPath(), fileName);
		return R.ok().data(fileName);
	}
}
