package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.*;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationDeptIndicatorItemService;
import com.ims.eval.service.IEvaluationDeptIndicatorService;
import com.ims.eval.service.IEvaluationDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 考评部门配置
 *
 * @author hlf
 * @date 2023/4/25 9:34
 * 文件说明：
 */
@RestController
@RequestMapping("//evaluation-dept")
public class EvaluationDeptController {

	@Autowired
	private IEvaluationDeptService evaluationDeptService;

	@Autowired
	private IEvaluationDeptIndicatorService evaluationDeptIndicatorService;

	@Autowired
	private IEvaluationDeptIndicatorItemService evaluationDeptIndicatorItemService;

	/**
	 * 考评部门配置列表信息（分页）
	 *
	 * @param pageNum  当前记录起始索引
	 * @param pageSize 每页显示记录数
	 * @param deptName 部门名称
	 * @param annual   年度
	 * @return 结果
	 */
	@GetMapping(value = "list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "deptName", required = false) String deptName,
		@RequestParam(value = "annual", required = false) String annual) {
		IPage<EvaluationDept> list = evaluationDeptService.listPage(pageNum, pageSize, deptName, annual);
		List<EvaluationDept> evaluationDeptList = list.getRecords();
		for (EvaluationDept evaluationDept : evaluationDeptList) {
			List<EvaluationDeptIndicator> evaluationDeptIndicatorList = evaluationDeptIndicatorService.selectEvaluationDeptIndicatorListByEvaluationDeptId(evaluationDept.getId());
			for (EvaluationDeptIndicator evaluationDeptIndicator : evaluationDeptIndicatorList) {
				List<EvaluationDeptIndicatorItem> evaluationDeptIndicatorItemList = evaluationDeptIndicatorItemService.selectEvaluationDeptIndicatorItemListByEvaluationDeptIndicatorId(evaluationDeptIndicator.getId());
				evaluationDeptIndicator.setEvaluationDeptIndicatorItemList(evaluationDeptIndicatorItemList);
			}
			evaluationDept.setEvaluationDeptIndicatorList(evaluationDeptIndicatorList);
		}
		return R.ok().data(list);
	}

	/**
	 * 考评部门配置信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "listAll")
	public R listAll() {
		List<EvaluationDept> list = evaluationDeptService.list();
		for (EvaluationDept evaluationDept : list) {
			List<EvaluationDeptIndicator> evaluationDeptIndicatorList = evaluationDeptIndicatorService.selectEvaluationDeptIndicatorListByEvaluationDeptId(evaluationDept.getId());
			for (EvaluationDeptIndicator evaluationDeptIndicator : evaluationDeptIndicatorList) {
				List<EvaluationDeptIndicatorItem> evaluationDeptIndicatorItemList = evaluationDeptIndicatorItemService.selectEvaluationDeptIndicatorItemListByEvaluationDeptIndicatorId(evaluationDeptIndicator.getId());
				evaluationDeptIndicator.setEvaluationDeptIndicatorItemList(evaluationDeptIndicatorItemList);
			}
			evaluationDept.setEvaluationDeptIndicatorList(evaluationDeptIndicatorList);
		}
		return R.ok().data(list);
	}

	/**
	 * 新增/修改考评部门配置信息
	 *
	 * @param dept 考评部门实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationDept dept) {
		try {
			boolean b = evaluationDeptService.saveOrUpdate(dept);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除考评部门配置信息
	 *
	 * @param ids
	 * @return
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b;
		b = evaluationDeptService.removeByIds(Arrays.asList(strings));
		for (String id : strings) {
			QueryWrapper<EvaluationDeptIndicator> qw1 = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(id)) {
				qw1.lambda().eq(EvaluationDeptIndicator::getEvaluationDeptId, id);
			}
			b = evaluationDeptIndicatorService.remove(qw1);
			QueryWrapper<EvaluationDeptIndicatorItem> qw2 = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(id)) {
				qw2.lambda().eq(EvaluationDeptIndicatorItem::getEvaluationDeptId, id);
			}
			b = evaluationDeptIndicatorItemService.remove(qw2);
		}
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

	/**
	 * 删除考评部门指标项信息
	 *
	 * @param id
	 * @return
	 */
	@PostMapping(value = "/remove/{id}")
	public R delete(@PathVariable("id") String id) {
		boolean b = evaluationDeptIndicatorItemService.removeById(id);
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}
}
