package com.ims.eval.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationRevision;
import com.ims.eval.entity.ResponsibilityIndicatorInfo;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationRevisionService;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import com.ims.eval.util.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 单位考评目标修订
 *
 * @author hlf
 * @date 2023/5/23 9:54
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-revision")
public class EvaluationRevisionController {

	@Autowired
	private IEvaluationRevisionService evaluationRevisionService;

	@Autowired
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;

	/**
	 * 单位考评目标修订列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param id                 主键
	 * @param responsibilityCode 业务编码
	 * @param cycleUnit          周期单位
	 * @param checkCycle         周期
	 * @param beginDate          考核开始时间
	 * @param endDate            考核截止时间
	 * @param stage              审批状态
	 * @param createBy           创建者
	 * @param year               年
	 * @param month              月
	 * @param des                描述
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
				  @RequestParam(value = "cycleUnit", required = false) String cycleUnit,
				  @RequestParam(value = "checkCycle", required = false) List<String> checkCycle,
				  @RequestParam(value = "beginDate", required = false) String beginDate,
				  @RequestParam(value = "endDate", required = false) String endDate,
				  @RequestParam(value = "stage", required = false) String stage,
				  @RequestParam(value = "createBy", required = false) String createBy,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month,
				  @RequestParam(value = "des", required = false) String des) {
		IPage<EvaluationRevision> list = evaluationRevisionService.listPage(pageNum, pageSize, id, responsibilityCode, cycleUnit, checkCycle, beginDate, endDate, stage, createBy, year, month, des);
		return R.ok().data(list);
	}

	/**
	 * 单位考评目标修订信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationRevision> list = evaluationRevisionService.list();
		return R.ok().data(list);
	}

	/**
	 * 考评修订
	 *
	 * @param id 考评目标主键
	 * @return 结果
	 */
	@PostMapping(value = "/revise/{id}")
	public R revise(@PathVariable String id) {
		try {
			boolean b = evaluationRevisionService.save(id);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情
	 *
	 * @param deptResponsibilityId 考评目标主键
	 * @return 结果
	 */
	@GetMapping(value = "/details/{deptResponsibilityId}")
	public R details(@PathVariable String deptResponsibilityId) {
		Map<String, List<ResponsibilityIndicatorInfoResDTO>> map = responsibilityIndicatorInfoService.selectResponsibilityIndicatorInfoList(deptResponsibilityId, null, "有效");
		return R.ok().data(map);
	}

	/**
	 * 详情-批量新增/修改
	 *
	 * @param responsibilityIndicatorInfoList 修改内容
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody List<ResponsibilityIndicatorInfo> responsibilityIndicatorInfoList) {
		try {
			boolean b = responsibilityIndicatorInfoService.saveOrUpdateBatch(responsibilityIndicatorInfoList);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-弹框数据
	 *
	 * @param deptResponsibilityId 考评目标主键
	 * @param isQuantified         是否量化
	 * @return 结果
	 */
	@GetMapping(value = "/popUpData")
	public R popUpData(@RequestParam(value = "deptResponsibilityId") String deptResponsibilityId,
					   @RequestParam(value = "isQuantified") String isQuantified) {
		List<ResponsibilityIndicatorInfoResDTO> list = responsibilityIndicatorInfoService.selectPopUpData(deptResponsibilityId, isQuantified, null);
		return R.ok().data(list);
	}

	/**
	 * 导入
	 *
	 * @param file         文件
	 * @param isQuantified 是否量化
	 * @return 结果
	 */
	@PostMapping(value = "/import")
	public R importData(@RequestParam("file") MultipartFile file, @RequestParam("isQuantified") String isQuantified) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<ResponsibilityIndicatorInfo> responsibilityIndicatorInfoList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					ResponsibilityIndicatorInfo responsibilityIndicatorInfo = new ResponsibilityIndicatorInfo();
					responsibilityIndicatorInfo.setId(str[0]);
					if ("是".equals(isQuantified)) {
						responsibilityIndicatorInfo.setQuantifiedValue(Double.parseDouble(str[7]));
						responsibilityIndicatorInfo.setState(str[9]);
					} else if ("否".equals(isQuantified)) {
						responsibilityIndicatorInfo.setNonQuantifiedValue(str[7]);
						responsibilityIndicatorInfo.setState(str[8]);
					}
					responsibilityIndicatorInfo.setUpdateTime(DateUtil.date());
					responsibilityIndicatorInfoList.add(responsibilityIndicatorInfo);
				}
				boolean b = responsibilityIndicatorInfoService.updateBatchById(responsibilityIndicatorInfoList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error().data("保存失败！");
				}
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.customError("上传文件为空！");
	}
}
