package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.entity.EvaluationWarningInfo;
import com.ims.eval.entity.EvaluationWarningRule;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationWarningInfoService;
import com.ims.eval.service.IEvaluationWarningRuleService;
import io.swagger.annotations.ApiOperation;
import javafx.beans.binding.SetExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-05-19
 */
@RestController
@RequestMapping("//evaluation/warning")
public class EvaluationWarningRuleController {

	@Autowired
	private IEvaluationWarningRuleService warningRuleService;
	@Autowired
	private IEvaluationWarningInfoService warningInfoService;


	@ApiOperation(value = "考评预警规则", notes = "查询")
	@GetMapping(value = "rule/list")
	public R EvaluationWarningRuleList(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "binSection", required = false) String binSection, //业务板块
		@RequestParam(value = "ruleName", required = false) String ruleName, //规则名称
		@RequestParam(value = "type", required = false) String type, //年度NDKP  季度JDKP  月度YDKP
		@RequestParam(value = "enable", required = false) Boolean enable) {
		IPage<EvaluationWarningRule> list = warningRuleService.warningRuleList(pageNum, pageSize, binSection, ruleName, type, enable);
		return R.ok().data(list);
	}


	@ApiOperation(value = "考评预警规则添加、更新", notes = "")
	@PostMapping(value = "rule/save")
	public R EvaluationWarningRuleSave(@RequestHeader(value = "Code", required = false) String code, @RequestBody EvaluationWarningRule warningRule){
		try {
			String decode = URLDecoder.decode(warningRule.getExpression(), "UTF-8");
			String expression = decode.replaceAll(" ", "+");
			warningRule.setExpression(expression);
			boolean flag = warningRuleService.saveOrUpdate(code, warningRule);
			if (flag) {
				return R.ok();
			}

		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			return R.error();
		}
		return R.error();
	}


	@ApiOperation(value = "考评预警规则删除", notes = "")
	@GetMapping(value = "rule/delete")
	public R EvaluationWarningRuleDelete(String id){
		boolean flag = warningRuleService.removeById(id);
		if (flag) {
			return R.ok();
		}
		return R.error();
	}


	@ApiOperation(value = "考评预警详情", notes = "查询")
	@GetMapping(value = "info/list")
	public R EvaluationWarningInfoList(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "binSection", required = false) String binSection, //业务板块
		@RequestParam(value = "st", required = false) String st, //开始时间
		@RequestParam(value = "et", required = false) String et) { //结束时间
		IPage<EvaluationWarningInfo> list = warningInfoService.warningInfoList(pageNum, pageSize, binSection, st, et);
		return R.ok().data(list);
	}


	@ApiOperation(value = "考评预警详情更新", notes = "")
	@PostMapping(value = "info/save")
	public R EvaluationWarningInfoSave(@RequestHeader("Code") String code, @RequestBody EvaluationWarningInfo warningInfo){
		boolean flag = warningInfoService.updateItem(code, warningInfo);
		if (flag) {
			return R.ok();
		}
		return R.error();
	}
}
