package com.ims.eval.controller;


import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.DataDictionary;
import com.ims.eval.entity.IndicatorType;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IIndicatorTypeService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 指标分类 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-09
 */
@RestController
@RequestMapping("//indicator-type")
public class IndicatorTypeController {

	@Autowired
	private IIndicatorTypeService iIndicatorTypeService;

	@Autowired
	private CacheContext cache;


	/**
	 * 查询
	 *
	 * @param id       主键ID
	 * @param type  type
	 * @param typeCode  code
	 * @return
	 */
	//@ImsPreAuth("eval:indicatorType:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "type", required = false) String type,
				  @RequestParam(value = "typeCode", required = false) String typeCode) {
//		List<IndicatorType> list = iIndicatorTypeService.list(id, typeName, typeCode);
		List<IndicatorType> list =null;
		if(null != type&&type.length()>0){
			list =CacheContext.itList.stream().filter(t->t.getType().equals(type)).collect(Collectors.toList());;
		}else{
			list =CacheContext.itList;
		}
		return R.ok().data(list);
	}


	/**
	 * 添加
	 *
	 * @param indicatorType
	 * @return
	 */

	//@ImsPreAuth("eval:indicatorType:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody IndicatorType indicatorType) {

		try {
			boolean b = iIndicatorTypeService.saveOrUpdate(indicatorType);
			if (b) {
				cache.initIndicatorType();
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:indicatorType:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = iIndicatorTypeService.removeByIds(Arrays.asList(strings));
		if (b) {
			cache.initDataDictionary();
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

}
