package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.MultipleBrand;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IMultipleBrandService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-05-22
 */
@RestController
@RequestMapping("//multiple-brand")
public class MultipleBrandController {

	@Autowired
	private IMultipleBrandService multipleBrandService;


	/**
	 * 查询
	 *
	 * @param id         主键ID
	 * @param parentId      父类id
	 * @return
	 */
		@GetMapping(value = "list")
	public R list(@RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "parentId", required = false) String parentId,
				  @RequestParam(value = "binSection", required = false) String  binSection,
				  @RequestParam(value = "checkCycle", required = false) String  checkCycle,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month) {

		List<MultipleBrand> list  =multipleBrandService.getMultipleBranList(id,parentId,binSection,checkCycle,year,month);
		return R.ok().data(list);
	}


	/**
	 *	 查询你
	 * @param pageNum 页数
	 * @param pageSize 页数大小
	 * @param id id
	 * @param parentId 父殴打
	 * @param binSection 板块
	 * @param checkCycle  周期
	 * @param year 年
	 * @param month 月
	 * @return
	 */
	@GetMapping(value = "tree")
	public R tree(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "parentId", required = false) String parentId,
				  @RequestParam(value = "binSection", required = false) String  binSection,
				  @RequestParam(value = "checkCycle", required = false) String  checkCycle,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month) {

		IPage<MultipleBrand> list  =multipleBrandService.getMultipleBranTree(pageNum,pageSize,id,parentId,binSection,checkCycle,year,month);
		return R.ok().data(list);
	}



	/**
	 * 添加
	 *
	 * @param multipleBrand
	 * @return
	 */

	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody MultipleBrand multipleBrand) {

		try {
			boolean b = multipleBrandService.saveOrUpdate(multipleBrand);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = multipleBrandService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

}
