package com.ims.eval.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationNotice;
import com.ims.eval.entity.OrganizationStructure;
import com.ims.eval.entity.custom.Path;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationNoticeService;
import com.ims.eval.service.IOrganizationStructureService;
import com.ims.eval.util.FileUploadUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 通告管理
 *
 * @author hlf
 * @date 2023/5/17 14:48
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-notice")
public class NoticeManagementController {

	@Autowired
	private IEvaluationNoticeService evaluationNoticeService;

	@Autowired
	private IOrganizationStructureService organizationStructureService;

	/**
	 * 通告管理列表信息（分页）
	 *
	 * @param pageNum     当前记录起始索引
	 * @param pageSize    每页显示记录数
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "noticeTitle", required = false) String noticeTitle) {
		IPage<EvaluationNotice> list = evaluationNoticeService.listPage(pageNum, pageSize, noticeTitle);
		List<EvaluationNotice> evaluationNoticeList = list.getRecords();
		for (EvaluationNotice obj : evaluationNoticeList) {
			StringBuilder names = new StringBuilder();
			if (!"".equals(obj.getSendToContent()) && null != obj.getSendToContent()) {
				String[] strings = obj.getSendToContent().split(",");
				for (String str : strings) {
					List<OrganizationStructure> deptList = organizationStructureService.getTree("23031001", 1, "140");
					if (deptList.size() > 0) {
						OrganizationStructure organizationStructure = deptList.stream().filter(item -> item.getId().equals(str)).findFirst().orElse(null);
						if (ObjectUtil.isNotNull(organizationStructure)) {
							names.append(organizationStructure.getName()).append(",");
						}
					}
				}
				obj.setDeptNames(names.toString().substring(0, names.toString().length() - 1));
			}
		}
		return R.ok().data(list);
	}

	/**
	 * 通告管理信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationNotice> list = evaluationNoticeService.list();
		return R.ok().data(list);
	}

	/**
	 * 新增/修改通告管理信息
	 *
	 * @param evaluationNotice 通告管理实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(EvaluationNotice evaluationNotice, @RequestParam(value = "files", required = false) MultipartFile[] files) throws IOException {
		try {
			boolean b = evaluationNoticeService.saveOrUpdate(evaluationNotice, files);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除通告管理信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = evaluationNoticeService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

	/**
	 * 修改状态
	 *
	 * @param ids          多主键
	 * @param releaseState 状态（未发布或已发布或草稿）
	 * @return 结果
	 */
	@PostMapping(value = "/modifiedState")
	public R modifiedState(@RequestParam(value = "ids") String ids,
						   @RequestParam(value = "releaseState") String releaseState) {
		boolean b = false;
		if (ids.length() > 0) {
			String[] strings = ids.split(",");
			for (String id : strings) {
				EvaluationNotice evaluationNotice = new EvaluationNotice();
				evaluationNotice.setId(id);
				if ("已发布".equals(releaseState)) {
					evaluationNotice.setReleaseDate(new Date());
				} else {
					evaluationNotice.setReleaseDate(null);
				}
				evaluationNotice.setReleaseState(releaseState);
				b = evaluationNoticeService.updateById(evaluationNotice);
			}
		}
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("失败！");
		}
	}

	/**
	 * 上传附件
	 *
	 * @param id    主键
	 * @param files 附件集
	 * @return 结果
	 * @throws IOException 异常
	 */
	@PostMapping(value = "/uploadAttachment")
	public R uploadAttachment(@RequestParam(value = "id") String id, @RequestParam(value = "files", required = false) MultipartFile[] files) throws IOException {
		try {
			boolean b = false;
			EvaluationNotice evaluationNotice = evaluationNoticeService.getById(id);
			StringBuilder fileNames = new StringBuilder();
			if (files.length > 0) {
				for (MultipartFile file : files) {
					// 上传文件路径
					String filePath = Path.getNoticePath();
					// 上传并返回新文件名称
					String fileName = FileUploadUtil.upload(filePath, file);
					fileNames.append(fileName).append(",");
				}
				String noticeAnnex = evaluationNotice.getNoticeAnnex() + "," + fileNames.toString().substring(0, fileNames.toString().length() - 1);
				evaluationNotice.setNoticeAnnex(noticeAnnex);
				b = evaluationNoticeService.updateById(evaluationNotice);
			}
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("上传附件失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 删除附件
	 *
	 * @param id          主键
	 * @param noticeAnnex 附件名称
	 * @return 结果
	 */
	@PostMapping(value = "/deleteAttachment")
	public R deleteAttachment(@RequestParam(value = "id") String id, @RequestParam(value = "noticeAnnex") String noticeAnnex) {
		try {
			boolean b;
			EvaluationNotice evaluationNotice = evaluationNoticeService.getById(id);
			String[] strings = evaluationNotice.getNoticeAnnex().split(",");
			StringBuilder fileNames = new StringBuilder();
			for (String str : strings) {
				if (str.substring(str.lastIndexOf("/") + 1, str.length()).equals(noticeAnnex)) {
					String filePath = Path.getStatementPath() + str;
					File file = new File(filePath);
					b = file.delete();
				} else {
					fileNames.append(str).append(",");
				}
			}
			evaluationNotice.setNoticeAnnex(fileNames.toString().substring(0, fileNames.toString().length() - 1));
			b = evaluationNoticeService.updateById(evaluationNotice);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("删除附件失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 通告展示
	 *
	 * @param deptId      部门主键
	 * @param noticeTitle 通告标题
	 * @return 结果
	 */
	@GetMapping(value = "/circularDisplay")
	public R circularDisplay(@RequestParam(value = "deptId") String deptId,
							 @RequestParam(value = "noticeTitle") String noticeTitle) {
		QueryWrapper<EvaluationNotice> qw = new QueryWrapper<>();
		qw.lambda().eq(EvaluationNotice::getReleaseState, "已发布");
		if (StringUtils.isNotEmpty(noticeTitle)) {
			qw.lambda().eq(EvaluationNotice::getNoticeTitle, noticeTitle);
		}
		List<EvaluationNotice> evaluationNoticeList = evaluationNoticeService.list(qw);
		List<EvaluationNotice> list = new ArrayList<>();
		for (EvaluationNotice evaluationNotice : evaluationNoticeList) {
			if ("所有单位".equals(evaluationNotice.getSendTo())) {
				if (evaluationNotice.getNoticeAnnex().length() > 0){
					String[] noticeAnnexList = evaluationNotice.getNoticeAnnex().split(",");
					StringBuilder noticeAnnex = new StringBuilder();
					for (String str : noticeAnnexList) {
						if (str.contains("All")) {
							noticeAnnex.append(str).append(",");
						} else if (str.contains(deptId)) {
							noticeAnnex.append(str).append(",");
						}
					}
					if (noticeAnnex.toString().length() > 0){
						evaluationNotice.setNoticeAnnex(noticeAnnex.toString().substring(0, noticeAnnex.toString().length() - 1));
					}
				}
				list.add(evaluationNotice);
			} else if ("个别部门".equals(evaluationNotice.getSendTo())) {
				String[] sendToContentList = evaluationNotice.getSendToContent().split(",");
				for (String str : sendToContentList) {
					if (str.equals(deptId)) {
						if (evaluationNotice.getNoticeAnnex().length() > 0){
							String[] noticeAnnexList = evaluationNotice.getNoticeAnnex().split(",");
							StringBuilder noticeAnnex = new StringBuilder();
							for (String str1 : noticeAnnexList) {
								if (str1.contains("All")) {
									noticeAnnex.append(str1).append(",");
								} else if (str1.contains(deptId)) {
									noticeAnnex.append(str1).append(",");
								}
							}
							if (noticeAnnex.toString().length() > 0){
								evaluationNotice.setNoticeAnnex(noticeAnnex.toString().substring(0, noticeAnnex.toString().length() - 1));
							}
						}
						list.add(evaluationNotice);
						break;
					}
				}
			} else if ("本部部门".equals(evaluationNotice.getSendTo())) {
				List<OrganizationStructure> deptList = organizationStructureService.getTree("23031001", 1, "140");
				if (deptList.size() > 0) {
					OrganizationStructure organizationStructure = deptList.stream().filter(item -> item.getId().equals(deptId)).findFirst().orElse(null);
					if (ObjectUtil.isNotNull(organizationStructure)) {
						list.add(evaluationNotice);
					}
				}
			}
		}
		return R.ok().data(list);
	}

	/**
	 * 文件预览
	 *
	 * @param url 文件路径
	 * @return 结果
	 */
	@GetMapping(value = "/filePreview")
	public R convertToHtml(@RequestParam("url") String url) throws UnknownHostException {
		InetAddress localHost = InetAddress.getLocalHost();
		String ip = localHost.getHostAddress();
		String httpUrl = "http://" + ip + ":28900/file/" + url;
		return R.ok().data(httpUrl);
	}
}
