package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.YearGroupCoefficient;
import com.ims.eval.entity.dto.request.YearGroupCoefficientDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IYearGroupCoefficientService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 年度经营业绩系数 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-19
 */
@RestController
@RequestMapping("//year-group-coefficient")
public class YearGroupCoefficientController {

	@Autowired
	private IYearGroupCoefficientService yearGroupCoefficientService;


	/**
	 * 查询分页数据
	 * @param pageNum
	 * @param pageSize
	 * @param id
	 * @param coefficientCode
	 * @param year
	 * @param createOrgName
	 * @param createBy
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "coefficientCode", required = false) String coefficientCode,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "createOrgName", required = false) String createOrgName,
				  @RequestParam(value = "createBy", required = false) String createBy) {
		IPage<YearGroupCoefficient> list = yearGroupCoefficientService.list(pageNum, pageSize, id, coefficientCode, year, createOrgName, createBy);
		return R.ok().data(list);
	}

	/**
	 * 查询所有数据
	 * @param id
	 * @param coefficientCode
	 * @param year
	 * @param createOrgName
	 * @param createBy
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "listAll")
	public R listAll(
		@RequestParam(value = "id", required = false) String id,
		@RequestParam(value = "coefficientCode", required = false) String coefficientCode,
		@RequestParam(value = "year", required = false) String year,
		@RequestParam(value = "createOrgName", required = false) String createOrgName,
		@RequestParam(value = "createBy", required = false) String createBy) {
		List<YearGroupCoefficient> list = yearGroupCoefficientService.listAll(id, coefficientCode, year, createOrgName, createBy);
		return R.ok().data(list);
	}




	/**
	 * 添加
	 *
	 * @param yearGroupCoefficient
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody YearGroupCoefficient yearGroupCoefficient) {

		try {
			boolean b = yearGroupCoefficientService.saveOrUpdate(yearGroupCoefficient);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 添加
	 *
	 * @param coefficientDTO
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/saveGroup")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R saveGroup(@RequestBody YearGroupCoefficientDTO coefficientDTO) {

		try {
			boolean b = yearGroupCoefficientService.saveGroup(coefficientDTO);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}





	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = yearGroupCoefficientService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}



	/**
	 * 查询byid
	 * @param id
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "listById")
	public R listById(@RequestParam(value = "id", required = true) String id) {
		YearGroupCoefficientDTO dto = yearGroupCoefficientService.getById( id);
		return R.ok().data(dto);
	}

}
