package com.ims.eval.schedule;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.FormulaUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.entity.EvaluationDeptBusinessAssessment;
import com.ims.eval.entity.EvaluationWarningInfo;
import com.ims.eval.entity.EvaluationWarningRule;
import com.ims.eval.entity.OrganizationEvaluation;
import com.ims.eval.service.IEvaluationDeptBusinessAssessmentService;
import com.ims.eval.service.IEvaluationWarningInfoService;
import com.ims.eval.service.IEvaluationWarningRuleService;
import com.ims.eval.service.IOrganizationEvaluationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 考评预警定时处理
 */
@Slf4j
@Component
public class EvaluationWarningSchedule {

	@Autowired
	private IEvaluationWarningRuleService warningRuleService;
	@Autowired
	private IEvaluationWarningInfoService warningInfoService;
	@Autowired
	private IOrganizationEvaluationService evaluationService;
	@Autowired
	private IEvaluationDeptBusinessAssessmentService assessmentService;


	//年度考评
	private final String TYPE_NDKP = "NDKP";
	//季度考评
	private final String TYPE_JDKP = "JDKP";
	//月度考评
	private final String TYPE_YDKP = "YDKP";
	//触发时间属于当前季度的第几个月，1,4,7,10表示每个季度的第一个月，2,5,8,11表示每个季度的第2个月，3,6,9,12表示每个季度的第三个月
	private final int[][] SEASON = new int[][]{{1,4,7,10},{2,5,8,11},{3,6,9,12}};


	/**
	 * 年度考评
	 */
	@Scheduled(cron = "0 0 23 * * ?") // cron表达式： 每天23点触发
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningRuleResolveNDKP(){
		List<EvaluationWarningRule> list = warningRuleService.warningRuleList(null, null, TYPE_NDKP, true);
		List<Map<String, Object>> companys = evaluationService.getCompanyWarning(TYPE_NDKP);
		if (list == null || list.size() == 0 || companys == null || companys.size() == 0) {
			return;
		}

		calculate(list, companys, TYPE_NDKP);
	}

	/**
	 * 季度考评
	 */
	@Scheduled(cron = "0 0 23 * * ?") // cron表达式： 每天23点触发
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningRuleResolveJDKP(){
		List<EvaluationWarningRule> list = warningRuleService.warningRuleList(null, null, TYPE_JDKP, true);
		List<Map<String, Object>> companys = evaluationService.getCompanyWarning(TYPE_JDKP);
		if (list == null || list.size() == 0) {
			return;
		}

		calculate(list, companys, TYPE_JDKP);
	}

	/**
	 * 月度考评
	 */
	@Scheduled(cron = "0 0 23 * * ?") // cron表达式： 每天23点触发
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningRuleResolveYDKP(){
		List<EvaluationWarningRule> list = warningRuleService.warningRuleList(null, null, TYPE_YDKP, true);
		List<Map<String, Object>> companys = evaluationService.getCompanyWarning(TYPE_YDKP);
		if (list == null || list.size() == 0) {
			return;
		}

		calculate(list, companys, TYPE_YDKP);
	}



	/**
	 * 单位年度考评
	 */
	@Scheduled(cron = "0 0 0 1 4 ?") // cron表达式： 每年4月1号0点触发
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningEvaluationNDKP(){
		Calendar cal = Calendar.getInstance();
		String year = String.valueOf(cal.get(Calendar.YEAR));
		List<OrganizationEvaluation> list = evaluationService.list(null, TYPE_NDKP, year, null);

		String company = "公司本部";
		String title = "单位年度考评";
		String content = "";
		if (list == null || list.size() == 0) {
			content = "单位年度考评计划未建立";
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
		Optional<OrganizationEvaluation> optional = list.stream().filter(item -> item.getStage().contains("未启动")).findFirst();
		if (optional.isPresent()){
			content = "单位年度考评流程未启动";
			company = StringUtils.isNotEmpty(optional.get().getCreateOrgName()) ? optional.get().getCreateOrgName() : company;
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
	}

	/**
	 * 单位季度考评
	 */
	@Scheduled(cron = "0 5 20 1 1,4,7,10 ?") // cron表达式： 每个季度的第一个月的20号的5执行一次
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningEvaluationJDKP(){
		Calendar cal = Calendar.getInstance();
		String year = String.valueOf(cal.get(Calendar.YEAR));
		String season = String.valueOf(DateUtils.getSeason(cal.getTime()));
		List<OrganizationEvaluation> list = evaluationService.list(null, TYPE_JDKP, year, season);

		String company = "公司本部";
		String title = "单位季度考评";
		String content = "";
		if (list == null || list.size() == 0) {
			content = "单位季度考评计划未建立";
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
		Optional<OrganizationEvaluation> optional = list.stream().filter(item -> item.getStage().contains("未启动")).findFirst();
		if (optional.isPresent()){
			content = "单位季度考评流程未启动";
			company = StringUtils.isNotEmpty(optional.get().getCreateOrgName()) ? optional.get().getCreateOrgName() : company;
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
	}

	/**
	 * 单位月度考评
	 */
	@Scheduled(cron = "0 0 7 5 * ?") // cron表达式： 每月5日上午7点触发
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningEvaluationYDKP(){
		Calendar cal = Calendar.getInstance();
		String year = String.valueOf(cal.get(Calendar.YEAR));
		String month = String.valueOf(cal.get(Calendar.MONTH) + 1);
		List<OrganizationEvaluation> list = evaluationService.list(null, TYPE_YDKP, year, month);

		String company = "公司本部";
		String title = "单位月度考评";
		String content = "";
		if (list == null || list.size() == 0) {
			content = "单位月度考评计划未建立";
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
		Optional<OrganizationEvaluation> optional = list.stream().filter(item -> item.getStage().contains("未启动")).findFirst();
		if (optional.isPresent()){
			content = "单位季度考评流程未启动";
			company = StringUtils.isNotEmpty(optional.get().getCreateOrgName()) ? optional.get().getCreateOrgName() : company;
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
	}


	/**
	 * 部门年度考评
	 */
	@Scheduled(cron = "0 0 0 1 4 ?") // cron表达式： 每年4月1号0点触发
//	@Scheduled(cron="0/50 * * * * ?")
	public void warningEvaluationDeptYDKP(){
		Calendar cal = Calendar.getInstance();
		String year = String.valueOf(cal.get(Calendar.YEAR));
		List<EvaluationDeptBusinessAssessment> list = assessmentService.list(year);

		String company = "公司部门";
		String title = "部门年度考评";
		String content = "";
		if (list == null || list.size() == 0) {
			content = "部门年度考评计划未建立";
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
		Optional<EvaluationDeptBusinessAssessment> optional = list.stream().filter(item -> item.getStage().contains("未启动")).findFirst();
		if (optional.isPresent()){
			content = "部门年度考评计划流程未启动";
			company = optional.isPresent() ? optional.get().getDeptName() : company;
			saveWarningInfo(null, null, title, content, null, null, null, company);
		}
	}



	private void calculate(List<EvaluationWarningRule> rules, List<Map<String, Object>> companys, String type){
		Map<Object, List<Map<String, Object>>> collect = companys.stream().collect(Collectors.groupingBy(m -> m.get("indicator_id")));
		for (EvaluationWarningRule rule : rules){
			if (!isFocus(rule, type)){
				continue;
			}
			String company = "";
			String targetValue = null;
			String completeValue = null;
			String expression = rule.getExpression();
			List<Map<String, Object>> list = collect.get(rule.getIndicatorId());
			for (Map<String, Object> mp : list){
				if (expression.contains("[" + mp.get("option_code") + "]")){
					//替换表达式数值
					expression.replace("[" + mp.get("option_code") + "]", String.valueOf(mp.get("quantified_value")));
				}
				targetValue = mp.containsKey("JHZ") ? (String) mp.get("JHZ") : mp.containsKey("DBZ") ? (String) mp.get("DBZ") : mp.containsKey("MBZ") ? (String) mp.get("MBZ") : null;
				completeValue = mp.containsKey("WCZ") ? (String) mp.get("WCZ") : null;
				company = mp.containsKey("organization_short_name") ? String.valueOf(mp.get("organization_short_name")) : company;
			}
			/**
			 * 0：未报警  1：报警
			 */
			double v = FormulaUtils.calculateFormula(expression);
			if (v >= 1){
				saveWarningInfo(rule.getBinSection(), rule.getIndicatorId(), rule.getRuleName(), rule.getRuleDes(), rule.getId(), targetValue, completeValue, company);
			}
		}
	}

	private void saveWarningInfo(String binSection, String indicatorId, String title,  String content,
								 String ruleId, String targetValue, String completeValue, String company){
		EvaluationWarningInfo info = new EvaluationWarningInfo();
		info.setBinSection(binSection);
		info.setIndicatorId(indicatorId);
		info.setTitle(title);
		info.setContent(content);
		info.setCreateTime(new Date());
		info.setRuleId(ruleId);
		info.setTargetValue(targetValue);
		info.setCompleteValue(completeValue);
		info.setCompany(company);
		warningInfoService.save(info);
	}


	private boolean isFocus(EvaluationWarningRule rule, String type){
		Calendar cal = Calendar.getInstance();
		int nowMonth = cal.get(Calendar.MONTH) + 1;
		int nowDay = cal.get(Calendar.DAY_OF_MONTH);
		cal.setTime(rule.getCronTime());
		switch (type){
			case TYPE_NDKP:
				int month = cal.get(Calendar.MONTH) + 1;
				int day = cal.get(Calendar.DAY_OF_MONTH);
				if (nowMonth == month && nowDay == day){
					return true;
				}
				break;
			case TYPE_JDKP:
				int day2 = cal.get(Calendar.DAY_OF_MONTH);
				int season = DateUtils.getSeason(cal.getTime());
				int index = season % 3;
				int[] arr = SEASON[index-1];
				for (int i = 0; i < arr.length; i++){
					if (nowMonth == arr[i] && nowDay == day2){
						return true;
					}
				}
				break;
			case TYPE_YDKP:
				int day3 = cal.get(Calendar.DAY_OF_MONTH);
				if (nowDay == day3){
					return true;
				}
				break;
		}
		return false;
	}


}
