package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.BinSection;
import com.ims.eval.entity.BinStage;
import com.ims.eval.dao.BinStageMapper;
import com.ims.eval.service.IBinStageService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 经营阶段分类 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-12
 */
@Service
public class BinStageServiceImpl extends ServiceImpl<BinStageMapper, BinStage> implements IBinStageService {

	@Override
	public List<BinStage> list(String id, String stageName, String stageCode) {

		QueryWrapper<BinStage> qw = new QueryWrapper();
		qw.lambda().eq(BinStage::getId, id);
		qw.lambda().like(BinStage::getStageName, stageName);
		qw.lambda().eq(BinStage::getStageCode, stageCode);
		qw.lambda().orderByAsc(BinStage::getOrderNum);
		List<BinStage> list = baseMapper.selectList(qw);

		return list;
	}

	@Transactional
	@Override
	public boolean saveOrUpdate(BinStage entity) {

		if (null != entity && (null != entity.getId() && !"".equals(entity.getId().trim()))) {
			BinStage stage = baseMapper.selectById(entity.getId());
			stage.setDelFlag(true);
			baseMapper.updateById(stage);
		}

		QueryWrapper<BinStage> qw = new QueryWrapper();
		qw.lambda().eq(BinStage::getStageCode, entity.getStageCode());
		qw.lambda().eq(BinStage::getDelFlag, false);
		List<BinStage> list = baseMapper.selectList(qw);
		if (null != list && list.size() > 0) {
			throw new CustomException("当前阶段分类已存在");
		}

		int count = baseMapper.selectCount(null);
		entity.setId(null);
		entity.setOrderNum(count);
		entity.setDelFlag(false);
		return super.saveOrUpdate(entity);
	}

}
