package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.dao.*;
import com.ims.eval.entity.CalculateIndicatorItemInfo;
import com.ims.eval.entity.OrganizationEvaluationInfo;
import com.ims.eval.entity.dto.response.OrganizationEvaluationInfoResDTO;
import com.ims.eval.entity.dto.response.OrganizationEvaluationResDTO;
import com.ims.eval.service.ICalculateIndicatorItemInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static org.codehaus.groovy.runtime.DefaultGroovyMethods.collect;

/**
 * <p>
 * 计算指标项明细 服务实现类
 * </p>
 *
 * @author xugp
 * @since 2023-03-26
 */
@Service
public class CalculateIndicatorItemInfoServiceImpl extends ServiceImpl<CalculateIndicatorItemInfoMapper, CalculateIndicatorItemInfo> implements ICalculateIndicatorItemInfoService {



	@Autowired
	private OrganizationEvaluationMapper organizationEvaluationMapper;

	@Autowired
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	@Autowired
	private IndicatorMapper indicatorMapper;


	@Override
	public int saveIndicatorInfo(List<OrganizationEvaluationInfo> organizationEvaluationInfos) throws Exception {

		List<CalculateIndicatorItemInfo> list = new ArrayList<>();

		OrganizationEvaluationResDTO organizationEvaluationResDTO = organizationEvaluationMapper.selectById(organizationEvaluationInfos.get(0).getOrganizationEvaluationId());
		//对考评规则id进行分组
		Map<String, List<OrganizationEvaluationInfo>> collect = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()) {
			if (entry.getValue().size() > 0) {
				//对指标id进行分组
				Map<String, List<OrganizationEvaluationInfo>> collectIn = entry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
				for (Map.Entry<String, List<OrganizationEvaluationInfo>> listEntry : collectIn.entrySet()) {
					List<OrganizationEvaluationInfo> listEntryValue = listEntry.getValue();
					for (OrganizationEvaluationInfo r : listEntryValue){
						if (r.getOptionCode().equals("CZ") || r.getOptionCode().equals("LRGXL") || r.getOptionCode().equals("DWQWLR")) {
							CalculateIndicatorItemInfo calculateIndicatorItemInfo = new CalculateIndicatorItemInfo();

							calculateIndicatorItemInfo.setIndicatorId(listEntry.getKey());
							calculateIndicatorItemInfo.setOptionCode(r.getOptionCode());
							calculateIndicatorItemInfo.setQuantifiedValue(r.getQuantifiedValue());
							calculateIndicatorItemInfo.setSectionId(indicatorMapper.selectById(listEntry.getKey()).getBinSection());
							calculateIndicatorItemInfo.setOrganizationType(organizationEvaluationResDTO.getOrganizationType());
							calculateIndicatorItemInfo.setCheckCycle(organizationEvaluationResDTO.getCheckCycle());
							calculateIndicatorItemInfo.setYear(organizationEvaluationResDTO.getYear());
							calculateIndicatorItemInfo.setMonth(organizationEvaluationResDTO.getMonth());
//							calculateIndicatorItemInfo.setOrganizationEvaluationId(organizationEvaluationResDTO.getId());
							list.add(calculateIndicatorItemInfo);
						}
					}
				}
			}
		}

		return saveCalculate(list);
	}

	@Override
	public int saveIndicatorInfoDTO(List<OrganizationEvaluationInfoResDTO> infoResDTOS) throws Exception {
		List<CalculateIndicatorItemInfo> list = new ArrayList<>();
		OrganizationEvaluationResDTO organizationEvaluationResDTO = organizationEvaluationMapper.selectById(infoResDTOS.get(0).getOrganizationEvaluationId());
		Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = infoResDTOS.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));

		for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()){
			//对指标id进行分组
			for (OrganizationEvaluationInfoResDTO r : childCodeGropListEntry.getValue()){
				if (r.getOptionCode().equals("CZ") ||  r.getOptionCode().equals("LRGXKH")|| r.getOptionCode().equals("LRGXL") || r.getOptionCode().equals("DWQWLR")) {
					CalculateIndicatorItemInfo calculateIndicatorItemInfo = new CalculateIndicatorItemInfo();
					calculateIndicatorItemInfo.setIndicatorId(r.getIndicatorId());
					calculateIndicatorItemInfo.setOptionCode(r.getOptionCode());
					calculateIndicatorItemInfo.setQuantifiedValue(r.getQuantifiedValue());
					calculateIndicatorItemInfo.setSectionId(r.getBinSection());
					calculateIndicatorItemInfo.setOrganizationType(organizationEvaluationResDTO.getOrganizationType());
					calculateIndicatorItemInfo.setCheckCycle(organizationEvaluationResDTO.getCheckCycle());
					calculateIndicatorItemInfo.setYear(organizationEvaluationResDTO.getYear());
					calculateIndicatorItemInfo.setMonth(organizationEvaluationResDTO.getMonth());
					calculateIndicatorItemInfo.setOrganizationEvaluationId(r.getOrganizationEvaluationId());
					calculateIndicatorItemInfo.setChildCode(r.getChildCode());
					list.add(calculateIndicatorItemInfo);
				}
			}
		}

		return saveCalculate(list);
	}


	@Transactional
	public int saveCalculate(List<CalculateIndicatorItemInfo> listall) throws Exception {

		if (listall.size() > 0){


			Map<String, List<CalculateIndicatorItemInfo>> childCodeGropList = listall.stream().collect(Collectors.groupingBy(CalculateIndicatorItemInfo::getChildCode));
			for (Map.Entry<String, List<CalculateIndicatorItemInfo>> childCodeGropListEntry : childCodeGropList.entrySet()){

				List<CalculateIndicatorItemInfo> list = childCodeGropListEntry.getValue();
				//遍历list解析最大值和最小值
				Map<String, List<CalculateIndicatorItemInfo>> collect = list.stream().collect(Collectors.groupingBy(CalculateIndicatorItemInfo::getOptionCode));
				//判断计算指标明细中之前是否存在记录 如果存在删除之前记录,重新新增
				List<CalculateIndicatorItemInfo> calculateIndicatorItemInfo = calculateIndicatorItemInfoMapper.selectList("","","","","",list.get(0).getOrganizationEvaluationId(),list.get(0).getChildCode());

				if (calculateIndicatorItemInfo.size() > 0){
					List<String> idList = calculateIndicatorItemInfo.stream().map(CalculateIndicatorItemInfo::getId).collect(Collectors.toList());
					calculateIndicatorItemInfoMapper.deleteBatchIds(idList);
				}
				for (Map.Entry<String, List<CalculateIndicatorItemInfo>> listEntry : collect.entrySet()){

					//从小到大的顺序排序
					Collections.sort(listEntry.getValue(), new Comparator<CalculateIndicatorItemInfo>() {
						@Override
						public int compare(CalculateIndicatorItemInfo u1, CalculateIndicatorItemInfo u2) {
							double diff = u1.getQuantifiedValue() - u2.getQuantifiedValue();
							BigDecimal two = new BigDecimal(diff);
							double three = two.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue();
							if (three > 0) {
								return 1;
							} else if (three < 0) {
								return -1;
							}
							return 0; //相等为0
						}
					});

					CalculateIndicatorItemInfo maxNum =  (CalculateIndicatorItemInfo)listEntry.getValue().get(listEntry.getValue().size()-1).clone();
					maxNum.setMark("1");

					CalculateIndicatorItemInfo minNum  = (CalculateIndicatorItemInfo)listEntry.getValue().get(0).clone();

					minNum.setMark("0");


					if (listEntry.getKey().equals("CZ")){
						//存储最大值
						maxNum.setOptionCode("CZMAX");
						minNum.setOptionCode("CZMIN");
					}else if (listEntry.getKey().equals("LRGXL")){
						maxNum.setOptionCode("LRGXLMAX");
						minNum.setOptionCode("LRGXLMIN");
					}else if (listEntry.getKey().equals("DWQWLR")){
						maxNum.setOptionCode("DWQWLRMAX");
						minNum.setOptionCode("DWQWLRMIN");
					}else if (listEntry.getKey().equals("LRGXKH")){
						maxNum.setOptionCode("LRGXKHMAX");
						minNum.setOptionCode("LRGXKHMIN");
					}


					int insert = calculateIndicatorItemInfoMapper.insert(maxNum);

					int insert1 = calculateIndicatorItemInfoMapper.insert(minNum);
					System.out.println(insert1);
				}
			}

		}else {
			throw new Exception("计算指标项明细为空");
		}
		return 1;
	}
}
