package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.DeptAssessmentDeclarationMapper;
import com.ims.eval.entity.DeptAssessmentDeclaration;
import com.ims.eval.entity.DeptAssessmentDeclarationContent;
import com.ims.eval.entity.dto.request.UserDTO;
import com.ims.eval.service.IDeptAssessmentDeclarationContentService;
import com.ims.eval.service.IDeptAssessmentDeclarationService;
import com.ims.eval.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hlf
 * @date 2023/5/29 15:01
 * 文件说明：
 */
@Service
public class DeptAssessmentDeclarationServiceImpl extends ServiceImpl<DeptAssessmentDeclarationMapper, DeptAssessmentDeclaration> implements IDeptAssessmentDeclarationService {

	@Autowired
	private IDeptAssessmentDeclarationContentService deptAssessmentDeclarationContentService;

	@Autowired
	private IUserService userService;

	@Override
	public IPage<DeptAssessmentDeclaration> listPage(Integer pageNum, Integer pageSize, String orderNumber, String deptId, String annual, String declarationMonth) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<DeptAssessmentDeclaration> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, orderNumber, deptId, annual, declarationMonth);
	}

	@Override
	public boolean save(DeptAssessmentDeclaration deptAssessmentDeclaration, HttpServletRequest request) {
		QueryWrapper<DeptAssessmentDeclaration> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getAnnual())) {
			qw.lambda().like(DeptAssessmentDeclaration::getAnnual, deptAssessmentDeclaration.getAnnual());
		}
		if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getDeclarationMonth())) {
			qw.lambda().eq(DeptAssessmentDeclaration::getDeclarationMonth, deptAssessmentDeclaration.getDeclarationMonth());
		}
		int count = super.count(qw);
		String id = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
		deptAssessmentDeclaration.setId(id);
		deptAssessmentDeclaration.setOrderNumber("KHSB_GDDL_" + deptAssessmentDeclaration.getAnnual() + "_" + StringUtils.addZeroForStr(String.valueOf(Integer.parseInt(deptAssessmentDeclaration.getDeclarationMonth())), 2, "l") + StringUtils.addZeroForStr(String.valueOf(count + 1), 2, "l"));
		deptAssessmentDeclaration.setStage("流程未启动");
		deptAssessmentDeclaration.setCreateTime(DateUtil.date());
		List<DeptAssessmentDeclarationContent> deptAssessmentDeclarationContentList = new ArrayList<>();
		JSONObject jsonArr = userService.pageList(1, 500, deptAssessmentDeclaration.getDeptId(), "", "", "", "", "", request);
		JSONObject jsonArr1 = (JSONObject) jsonArr.get("data");
		JSONArray array = JSONUtil.parseArray(jsonArr1.get("records"));
		System.out.println(array);
		List<UserDTO> userList = JSONUtil.toList(array, UserDTO.class);
		for (int i = 0; i < userList.size(); i++) {
			DeptAssessmentDeclarationContent deptAssessmentDeclarationContent = new DeptAssessmentDeclarationContent();
			deptAssessmentDeclarationContent.setAssessmentDeclarationId(id);
			deptAssessmentDeclarationContent.setSerialNumber(i + 1);
			deptAssessmentDeclarationContent.setEmployeeId(userList.get(i).getId());
			deptAssessmentDeclarationContent.setEmployeeNo(userList.get(i).getNo());
			deptAssessmentDeclarationContent.setEmployeeName(userList.get(i).getName());
			deptAssessmentDeclarationContent.setSuggestedValue("1");
			deptAssessmentDeclarationContentList.add(deptAssessmentDeclarationContent);
		}
		boolean b = super.save(deptAssessmentDeclaration);
		if (b) {
			b = deptAssessmentDeclarationContentService.saveBatch(deptAssessmentDeclarationContentList);
		}
		return b;
	}

	@Override
	public boolean removeByIds(List<String> ids) {
		boolean b = false;
		for (String id : ids) {
			DeptAssessmentDeclaration deptAssessmentDeclaration = super.getById(id);
			if ("流程未启动".equals(deptAssessmentDeclaration.getStage()) || "流程已结束".equals(deptAssessmentDeclaration.getStage())) {
				b = deptAssessmentDeclarationContentService.removeByAssessmentDeclarationId(id);
				if (b) {
					super.removeById(id);
				}
			} else {
				return b;
			}
		}
		return b;
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		DeptAssessmentDeclaration deptAssessmentDeclaration = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			deptAssessmentDeclaration.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			deptAssessmentDeclaration.setInstId(instId);
		}
		return super.saveOrUpdate(deptAssessmentDeclaration);
	}

}
