package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.DoubleUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.*;
import com.ims.eval.dao.EvaluateReportMapper;
import com.ims.eval.entity.dto.response.EvaluateReportInfoResDTO;
import com.ims.eval.entity.dto.response.EvaluateRuleInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-25
 */
@Service
public class EvaluateReportServiceImpl extends ServiceImpl<EvaluateReportMapper, EvaluateReport> implements IEvaluateReportService {


	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;


	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Autowired
	private IEvaluateRuleService evaluateRuleService;


	@Autowired
	private IEvaluationScoreCountService evaluationScoreCountService;

	@Autowired
	private IEvaluateReportInfoService evaluateReportInfoService;

	@Autowired
	private IMultipleBrandService multipleBrandService;


	@Autowired
	private CacheContext cache;

	@Autowired
	private IEvaluationPortalService evaluationPortalService;

	@Autowired
	private IYearOperatingCoefficientService yearOperatingCoefficientService;


	@Autowired
	private  IPartyBuildingMultiplierService partyBuildingMultiplierService;
	//考评结果报告
	//按照业务类型生成
	//业务阶综合展示



	@Transactional
	@Override
	public boolean generateEvaluateReport(String organizationEvaluationId, boolean backroll) {


		//业务
		List<BinSection> sections = CacheContext.bsnList;

		OrganizationEvaluation evaluation = organizationEvaluationService.getById(organizationEvaluationId);
		if (null == evaluation) {
			throw new CustomException("没有此靠考评");
		}
		//获取到对应组织考评规则
		String evaluationRuleId = evaluation.getOrganizationEvaluationRuleId();
		if (null == evaluationRuleId || evaluationRuleId.length() <= 0) {
			throw new CustomException("组织考评id为空");
		}
		String month = "";
		//获取单位配置规则
		List<String> evaluationRuleIds = Arrays.asList(evaluationRuleId.split(","));
		for (BinSection bi : sections) {


			EvaluateReport evaluateReport = new EvaluateReport();
			StringBuilder reportName = new StringBuilder();
			if ("NDKP".equals(evaluation.getCheckCycle())) {
				if("FD".equals(bi.getSectionCode()) || "GD".equals(bi.getSectionCode())){
					continue;
				}
				reportName.append("年度-");
				month = "";
				evaluateReport.setMonth("-");
			}
			if ("JDKP".equals(evaluation.getCheckCycle())) {
				reportName.append("季度-");
				evaluateReport.setMonth(evaluation.getMonth());
				if("XNYFG".equals(bi.getSectionCode()) || "ZHZC".equals(bi.getSectionCode())||
					"GCGS".equals(bi.getSectionCode()) || "TJGS".equals(bi.getSectionCode())||
					"QQ".equals(bi.getSectionCode()) || "JJ".equals(bi.getSectionCode())||
					"GD".equals(bi.getSectionCode())){
					continue;
				}

			}
			if ("YDKP".equals(evaluation.getCheckCycle())) {
				if("XNYFG".equals(bi.getSectionCode()) || "ZHZC".equals(bi.getSectionCode())||
					"GCGS".equals(bi.getSectionCode()) || "TJGS".equals(bi.getSectionCode())||
					"QQ".equals(bi.getSectionCode()) || "JJ".equals(bi.getSectionCode())||
					"GD".equals(bi.getSectionCode())){
					continue;
				}
				reportName.append("月度-");
				month = evaluation.getMonth();
				evaluateReport.setMonth(evaluation.getMonth());
			}
			reportName.append(bi.getSectionName());
			if ("DWKP".equals(evaluation.getOrganizationType())) {
				reportName.append("单位");
			}
			if ("BNMKP".equals(evaluation.getOrganizationType())) {
				reportName.append("部门");
			}
			reportName.append("综合考核评分汇总表");

			evaluateReport.setEvaluateReportName(reportName.toString());
			evaluateReport.setOrganizationEvaluationId(evaluation.getId());
			evaluateReport.setBinSection(bi.getId());
			evaluateReport.setBinSectionName(bi.getSectionName());
			evaluateReport.setOrganizationType(evaluation.getOrganizationType());
			evaluateReport.setCheckCycle(evaluation.getCheckCycle());
			evaluateReport.setYear(evaluation.getYear());

			evaluateReport.setCreateTime(new Date());
			evaluateReport.setCreateBy("");//人员
			List<EvaluateReport> oriReports = list(evaluation.getId(), bi.getId());
			if (oriReports.size() > 0 && !backroll) {
				for(EvaluateReport er : oriReports){
					List<EvaluateReportInfo> rlist = evaluateReportInfoService.listByReportId(er.getId());

					if(null !=rlist && rlist.size()>0){
						boolean rlistb =  evaluateReportInfoService.removeByIds(rlist.stream().map(EvaluateReportInfo::getId).collect(Collectors.toList()));
						if (!rlistb) {
							throw new CustomException("综合考核评分汇总计算失败");
						}
					}
				}
				boolean oriReportsb = this.removeByIds(oriReports.stream().map(EvaluateReport::getId).collect(Collectors.toList()));
				if (!oriReportsb) {
					throw new CustomException("综合考核评分汇总计算失败");
				}
				//throw new CustomException("综合考核评分汇总已生成");
			} else {
				for (EvaluateReport r : oriReports) {
					boolean b = super.removeById(r.getId());
					if (!b) {
						throw new CustomException("综合考核评分汇总重算失败");
					}
					List<EvaluateReportInfo> infolist = evaluateReportInfoService.listByReportId(r.getId());
					if (null != infolist && infolist.size() > 0) {
						List<String> infolistIds = infolist.stream().map(EvaluateReportInfo::getId).collect(Collectors.toList());
						b = evaluateReportInfoService.removeByIds(infolistIds);
						if (!b) {
							throw new CustomException("综合考核评分汇总重算失败");
						}
					}
				}
			}
			boolean b = super.saveOrUpdate(evaluateReport);

			if (!b) {
				throw new CustomException("综合考核评分汇总表存储失败");
			}


			//获取当前模块得公司
			List<OrganizationEvaluationRule> ruleList = organizationEvaluationRuleService.getOrganizationEvaluationRuleByIds(evaluationRuleIds, bi.getId());
			//各板块明细存储
			List<EvaluateReportInfo> evaluateRuleInfoList = new ArrayList<>();
			for (OrganizationEvaluationRule rule : ruleList) {



				StringBuilder remark = new StringBuilder();
				//生产95%基建5%前期0%
				remark.append("生产").append(rule.getScjyWeight() * 100).append("%");
				remark.append("基建").append(rule.getJjWeight() * 100).append("%");
				remark.append("前期").append(rule.getQqWeight() * 100).append("%");


				//存储多块牌子数据
				List<MultipleBrand> brands = multipleBrandService.getMultipleBranList2(rule.getOrganizationId(), "0", bi.getId(), evaluation.getCheckCycle(), evaluation.getYear(), month);

				Double totalProfit = 0.00;//累加收入
				Double spMultiply = 0.00;//收入
				Double scjyScore = 0.0;//分数
				if (null != brands && brands.size() > 0) {
					for (MultipleBrand mb : brands) {
						//累加收入
						totalProfit = totalProfit + mb.getProfit();
						//累加收入和分数的积
						spMultiply = spMultiply + mb.getProfit() * mb.getScore();

						List<MultipleBrand> parentBrands = multipleBrandService.getMultipleBranList2("", mb.getOrganizationId(), bi.getId(), evaluation.getCheckCycle(), evaluation.getYear(), month);
						for (MultipleBrand pb : parentBrands) {
							//累加收入
							totalProfit = totalProfit + pb.getProfit();
							spMultiply = spMultiply + mb.getProfit() * mb.getScore();
						}

					}
					scjyScore = spMultiply / totalProfit;
				}


				EvaluateReportInfo reportInfo = new EvaluateReportInfo();
				reportInfo.setEvaluateReportId(evaluateReport.getId());
				reportInfo.setOrganizationId(rule.getOrganizationId());
				reportInfo.setOrganizationName(rule.getOrganizationShortName());

				//生产经营得分
				if(scjyScore>0){
					reportInfo.setScjyScore(DoubleUtils.keepPrecision(scjyScore, 2));
				}else {
					List<EvaluationScoreCount> scjyScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("SCJY".split(",")));
					scjyScore = scjyScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
					reportInfo.setScjyScore(DoubleUtils.keepPrecision(scjyScore, 2));
				}

				//前期得分
				List<EvaluationScoreCount> qqScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("QQXM".split(",")));
				Double qqScore = qqScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setQqScore(DoubleUtils.keepPrecision(qqScore, 2));

				//基建得分
				List<EvaluationScoreCount> jjScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("JJXM".split(",")));
				Double jjScore = jjScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setJjScore(DoubleUtils.keepPrecision(jjScore, 2));


				List<EvaluationScoreCount> addSubScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("ZDZX,GLSX".split(",")));
				Double addSubScore = addSubScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				//加分项
				double addScore = addSubScoreCount.stream().filter(a->a.getScore()>0).collect(Collectors.toList()).stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setAddScore(DoubleUtils.keepPrecision(addScore, 2));

				//减分项
				double subScore = addSubScoreCount.stream().filter(a->a.getScore()<0).collect(Collectors.toList()).stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setSubScore(DoubleUtils.keepPrecision(subScore, 2));

				//加减分合计
				reportInfo.setAddSubScore(DoubleUtils.keepPrecision(addSubScore, 2));

				//综合得分
				//经营分*比例 + 基建分*比例 + 前期分*比例 + 加减分合计
				Double zhScore = scjyScore*rule.getScjyWeight() + qqScore*rule.getQqWeight() + jjScore*rule.getJjWeight() + addSubScore;
				reportInfo.setZhScore(DoubleUtils.keepPrecision(zhScore, 2));
				//年度
				if("NDKP".equals(rule.getEvaluationCycle())){

					List<YearOperatingCoefficient> list = yearOperatingCoefficientService.getListByOrganizationId(rule.getOrganizationId(), rule.getBinSection(),evaluation.getYear());
					if(null != list && list.size()>0){
						//经营业绩系数
						reportInfo.setJyyjXs((null != list && list.size()>0) ? list.get(0).getK(): 1);
						//关键业绩得分
						//综合得分*经营业绩系数 +加扣分
						double gjyjXs = zhScore * list.get(0).getK() + addSubScore;
						reportInfo.setGjyjXs(gjyjXs);
					}else {
						//经营业绩系数
						reportInfo.setJyyjXs(1);
						//关键业绩系数
						reportInfo.setGjyjXs(1);
					}
					List<PartyBuildingMultiplier> multiplierList = partyBuildingMultiplierService.listByParam(null, rule.getOrganizationId(), null, evaluation.getYear());
					if(null != multiplierList && multiplierList.size()>0){
						//党建考核乘数
						reportInfo.setDjkhCs(Double.valueOf(multiplierList.get(0).getMultiplier()));
					}else {
						//党建考核乘数
						reportInfo.setDjkhCs(1);
					}
				}else {
					reportInfo.setScore(DoubleUtils.keepPrecision(zhScore, 2));
					reportInfo.setDiscountScore(1.0);
				}
				reportInfo.setRemark(remark.toString());
				evaluateRuleInfoList.add(reportInfo);
			}

			for (EvaluateReportInfo info : evaluateRuleInfoList) {
				b = evaluateReportInfoService.saveOrUpdate(info);
				if (!b) {
					throw new CustomException("综合考核评分汇总明细存储失败");
				}
			}
		}
		return true;
	}


	@Override
	public IPage<EvaluateReport> list(Integer pageNum, Integer pageSize, String id, String evaluateReportName, String binSection, String organizationEvaluationId, String organizationType, String checkCycle, String year, String month) {
		Page<Indicator> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		IPage<EvaluateReport> list = null;
		try {
			list = baseMapper.selectListPage(page, id, evaluateReportName, binSection, organizationEvaluationId, organizationType, checkCycle, year, month);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return list;
	}

	@Override
	public List<EvaluateReport> list(String organizationEvaluationId, String binSection) {
		QueryWrapper<EvaluateReport> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(EvaluateReport::getOrganizationEvaluationId, organizationEvaluationId);
		}
		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(EvaluateReport::getBinSection, binSection);
		}
		List<EvaluateReport> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public List<EvaluateReportInfoResDTO> list(String organizationEvaluationId, String binSection, String checkCycle, String year, String month) {



		return null;
	}


}

