package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Dict;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.EvaluationDeptBusinessPlanMapper;
import com.ims.eval.entity.EvaluationDeptBusinessAssessment;
import com.ims.eval.entity.EvaluationDeptBusinessPlan;
import com.ims.eval.service.IEvaluationDeptBusinessAssessmentService;
import com.ims.eval.service.IEvaluationDeptBusinessContentService;
import com.ims.eval.service.IEvaluationDeptBusinessPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author hlf
 * @date 2023/4/27 10:33
 * 文件说明：
 */
@Service
public class EvaluationDeptBusinessPlanServiceImpl extends ServiceImpl<EvaluationDeptBusinessPlanMapper, EvaluationDeptBusinessPlan> implements IEvaluationDeptBusinessPlanService {

	@Autowired
	private IEvaluationDeptBusinessContentService evaluationDeptBusinessContentService;

	@Autowired
	private IEvaluationDeptBusinessAssessmentService evaluationDeptBusinessAssessmentService;

	@Override
	public IPage<EvaluationDeptBusinessPlan> listPage(Integer pageNum, Integer pageSize, String responsibilityCode, String des, String appraisalYear, String stage, String deptId) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluationDeptBusinessPlan> page = new Page<>(pageNum, pageSize);
		if ("23031009".equals(deptId)){
			deptId = "";
		}
		return baseMapper.selectListPage(page, responsibilityCode, des, appraisalYear, stage, deptId);
	}

	@Override
	public boolean save(EvaluationDeptBusinessPlan evaluationDeptBusinessPlan) {
		evaluationDeptBusinessPlan.setId(Convert.toStr(IdUtil.getSnowflake(1, 1).nextId()));
		evaluationDeptBusinessPlan.setResponsibilityCode("YJZB_GDDL_BMKP_" + evaluationDeptBusinessPlan.getAppraisalYear());
		evaluationDeptBusinessPlan.setDocumentStatus("有效");
		evaluationDeptBusinessPlan.setStage("流程未启动");
		evaluationDeptBusinessPlan.setCreateTime(DateUtil.date());
		JSONArray array = JSONUtil.parseArray(evaluationDeptBusinessPlan.getDeptData());
		List<Dict> deptList = JSONUtil.toList(array, Dict.class);
		StringBuilder deptIds = new StringBuilder();
		StringBuilder deptNames = new StringBuilder();
		for (Dict dict : deptList) {
			deptIds.append(dict.get("id").toString()).append(",");
			deptNames.append(dict.get("keyName").toString()).append(",");
		}
		evaluationDeptBusinessPlan.setDeptId(deptIds.toString().substring(0, deptIds.toString().length() - 1));
		evaluationDeptBusinessPlan.setDeptName(deptNames.toString().substring(0, deptNames.toString().length() - 1));
		return super.save(evaluationDeptBusinessPlan);
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			if ("组织人事部领导审核".equals(state)) {
				state = "流程已结束";
				EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = new EvaluationDeptBusinessAssessment();
				evaluationDeptBusinessAssessment.setId(evaluationDeptBusinessPlan.getId());
				evaluationDeptBusinessAssessment.setResponsibilityCode(evaluationDeptBusinessPlan.getResponsibilityCode());
				evaluationDeptBusinessAssessment.setDeptId(evaluationDeptBusinessPlan.getDeptId());
				evaluationDeptBusinessAssessment.setDeptName(evaluationDeptBusinessPlan.getDeptName());
				evaluationDeptBusinessAssessment.setDeptCode(evaluationDeptBusinessPlan.getDeptCode());
				evaluationDeptBusinessAssessment.setAppraisalYear(evaluationDeptBusinessPlan.getAppraisalYear());
				evaluationDeptBusinessAssessment.setEvaluationCategory(evaluationDeptBusinessPlan.getEvaluationCategory());
				evaluationDeptBusinessAssessment.setBusinessClass(evaluationDeptBusinessPlan.getBusinessClass());
				evaluationDeptBusinessAssessment.setDocumentStatus(evaluationDeptBusinessPlan.getDocumentStatus());
				evaluationDeptBusinessAssessment.setStage("流程未启动");
				evaluationDeptBusinessAssessment.setDes(evaluationDeptBusinessPlan.getDes());
				evaluationDeptBusinessAssessment.setReviewPeople(evaluationDeptBusinessPlan.getReviewPeople());
				evaluationDeptBusinessAssessment.setReviewPeopleTime(evaluationDeptBusinessPlan.getReviewPeopleTime());
				evaluationDeptBusinessAssessment.setReviewOpinion(evaluationDeptBusinessPlan.getReviewOpinion());
				evaluationDeptBusinessAssessment.setFinalReviewPeople(evaluationDeptBusinessPlan.getFinalReviewPeople());
				evaluationDeptBusinessAssessment.setFinalReviewPeopleTime(evaluationDeptBusinessPlan.getFinalReviewPeopleTime());
				evaluationDeptBusinessAssessment.setFinalReviewOpinion(evaluationDeptBusinessPlan.getFinalReviewOpinion());
				evaluationDeptBusinessAssessment.setCreateTime(DateUtil.date());
				evaluationDeptBusinessAssessment.setCreateBy(evaluationDeptBusinessPlan.getCreateBy());
				evaluationDeptBusinessAssessment.setUpdateTime(evaluationDeptBusinessPlan.getUpdateTime());
				evaluationDeptBusinessAssessment.setUpdateBy(evaluationDeptBusinessPlan.getUpdateBy());
				evaluationDeptBusinessAssessment.setRemark(evaluationDeptBusinessPlan.getRemark());
				evaluationDeptBusinessAssessmentService.save(evaluationDeptBusinessAssessment);
			}
			evaluationDeptBusinessPlan.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			evaluationDeptBusinessPlan.setInstId(instId);
		}
		return super.saveOrUpdate(evaluationDeptBusinessPlan);
	}

	@Override
	public boolean removeByIds(List<String> ids) {
		boolean b = false;
		for (String id : ids) {
			EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = super.getById(id);
			if ("流程未启动".equals(evaluationDeptBusinessPlan.getStage()) || "流程已结束".equals(evaluationDeptBusinessPlan.getStage())) {
				b = super.removeById(id);
				if (b) {
					evaluationDeptBusinessAssessmentService.removeById(id);
					evaluationDeptBusinessContentService.removeByBusinessPlanId(id);
				}
			} else {
				return b;
			}
		}
		return b;
	}

	@Override
	public boolean updateById(EvaluationDeptBusinessPlan evaluationDeptBusinessPlan) {
		evaluationDeptBusinessPlan.setUpdateTime(DateUtil.date());
		EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = new EvaluationDeptBusinessAssessment();
		evaluationDeptBusinessAssessment.setId(evaluationDeptBusinessPlan.getId());
		evaluationDeptBusinessAssessment.setDes(evaluationDeptBusinessPlan.getDes());
		evaluationDeptBusinessAssessmentService.updateById(evaluationDeptBusinessAssessment);
		return super.updateById(evaluationDeptBusinessPlan);
	}
}
