package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.entity.*;
import com.ims.eval.dao.EvaluationDeptMapper;
import com.ims.eval.service.IEvaluationDeptIndicatorItemService;
import com.ims.eval.service.IEvaluationDeptIndicatorService;
import com.ims.eval.service.IEvaluationDeptService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * <p>
 * 考评部门 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-06
 */
@Service
public class EvaluationDeptServiceImpl extends ServiceImpl<EvaluationDeptMapper, EvaluationDept> implements IEvaluationDeptService {

	@Autowired
	private IEvaluationDeptIndicatorService evaluationDeptIndicatorService;

	@Autowired
	private IEvaluationDeptIndicatorItemService evaluationDeptIndicatorItemService;

	@Override
	public IPage<EvaluationDept> listPage(Integer pageNum, Integer pageSize, String deptName, String annual) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluationDept> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, deptName, annual);
	}

	@Transactional
	@Override
	public boolean saveOrUpdate(EvaluationDept dept) {
		if (StringUtils.isEmpty(dept.getId())){
			QueryWrapper<EvaluationDept> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(dept.getDeptId())) {
				qw.lambda().eq(EvaluationDept::getDeptId, dept.getDeptId());
			}
			if (StringUtils.isNotEmpty(dept.getAnnual())) {
				qw.lambda().eq(EvaluationDept::getAnnual, dept.getAnnual());
			}
			EvaluationDept evaluationDept = baseMapper.selectOne(qw);
			if (ObjectUtil.isNotNull(evaluationDept)){
				return false;
			}
		}
		boolean b;
		if (StringUtils.isEmpty(dept.getId())) {
			String edId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
			dept.setId(edId);
			//默认启用
			dept.setEnable(true);
			//新增部门配置信息
			b = super.save(dept);
			for (EvaluationDeptIndicator evaluationDeptIndicator : dept.getEvaluationDeptIndicatorList()) {
				if (evaluationDeptIndicator.getEvaluationDeptIndicatorItemList().size() > 0) {
					String ediId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
					evaluationDeptIndicator.setId(ediId);
					evaluationDeptIndicator.setEvaluationDeptId(edId);
					//新增部门指标信息
					b = evaluationDeptIndicatorService.save(evaluationDeptIndicator);
					for (EvaluationDeptIndicatorItem evaluationDeptIndicatorItem : evaluationDeptIndicator.getEvaluationDeptIndicatorItemList()) {
						String ediiId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
						evaluationDeptIndicatorItem.setId(ediiId);
						evaluationDeptIndicatorItem.setEvaluationDeptId(edId);
						evaluationDeptIndicatorItem.setEvaluationDeptIndicatorId(ediId);
						evaluationDeptIndicatorItem.setIndicatorName(evaluationDeptIndicator.getIndicatorName());
						evaluationDeptIndicatorItem.setIndicatorCode(evaluationDeptIndicator.getIndicatorCode());
						//新增部门指标项信息
						b = evaluationDeptIndicatorItemService.save(evaluationDeptIndicatorItem);
					}
				}
			}
		} else {
			//默认启用
			dept.setEnable(true);
			b = super.updateById(dept);
			Iterator<EvaluationDeptIndicator> iterator = dept.getEvaluationDeptIndicatorList().iterator();
			while (iterator.hasNext()) {
				EvaluationDeptIndicator evaluationDeptIndicator = iterator.next();
				//是否有指标项
				if (evaluationDeptIndicator.getEvaluationDeptIndicatorItemList().size() > 0) {
					//是否有主键
					if (StringUtils.isEmpty(evaluationDeptIndicator.getId())) {
						String ediId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
						evaluationDeptIndicator.setId(ediId);
						evaluationDeptIndicator.setEvaluationDeptId(dept.getId());
						//新增部门指标信息
						b = evaluationDeptIndicatorService.save(evaluationDeptIndicator);
						for (EvaluationDeptIndicatorItem evaluationDeptIndicatorItem : evaluationDeptIndicator.getEvaluationDeptIndicatorItemList()) {
							String ediiId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
							evaluationDeptIndicatorItem.setId(ediiId);
							evaluationDeptIndicatorItem.setEvaluationDeptId(dept.getId());
							evaluationDeptIndicatorItem.setEvaluationDeptIndicatorId(ediId);
							evaluationDeptIndicatorItem.setIndicatorName(evaluationDeptIndicator.getIndicatorName());
							evaluationDeptIndicatorItem.setIndicatorCode(evaluationDeptIndicator.getIndicatorCode());
							//新增部门指标项信息
							b = evaluationDeptIndicatorItemService.save(evaluationDeptIndicatorItem);
						}
					} else {
						b = evaluationDeptIndicatorService.updateById(evaluationDeptIndicator);
						for (EvaluationDeptIndicatorItem evaluationDeptIndicatorItem : evaluationDeptIndicator.getEvaluationDeptIndicatorItemList()) {
							if (StringUtils.isEmpty(evaluationDeptIndicatorItem.getId())) {
								String ediiId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
								evaluationDeptIndicatorItem.setId(ediiId);
								evaluationDeptIndicatorItem.setEvaluationDeptId(dept.getId());
								evaluationDeptIndicatorItem.setEvaluationDeptIndicatorId(evaluationDeptIndicator.getId());
								evaluationDeptIndicatorItem.setIndicatorName(evaluationDeptIndicator.getIndicatorName());
								evaluationDeptIndicatorItem.setIndicatorCode(evaluationDeptIndicator.getIndicatorCode());
								//新增部门指标信息
								b = evaluationDeptIndicatorItemService.save(evaluationDeptIndicatorItem);
							} else {
								b = evaluationDeptIndicatorItemService.updateById(evaluationDeptIndicatorItem);
							}
						}
					}
				} else {
					boolean flag = evaluationDeptIndicatorService.removeById(evaluationDeptIndicator.getId());
					if (flag) {
						iterator.remove();
					}
				}
			}
		}
		return b;
	}
}
