package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.*;
import com.ims.eval.entity.*;
import com.ims.eval.service.IEvaluationScoringRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 考评得分规则表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-27
 */
@Service
public class EvaluationScoringRuleServiceImpl extends ServiceImpl<EvaluationScoringRuleMapper, EvaluationScoringRule> implements IEvaluationScoringRuleService {


	@Autowired
	private EvaluationScoringRuleMapper evaluationScoringRuleMapper;

	@Autowired
	private IntervalScoringTableMapper intervalScoringTableMapper;

	@Autowired
	private IndicatorMapper indicatorMapper;

	@Autowired
	private BinSectionMapper binSectionMapper;


	@Override
	public IPage<EvaluationScoringRule> list(Integer pageNum, Integer pageSize,String indicatorId,String scoreRuleName,String businessPhase){

		QueryWrapper<EvaluationScoringRule> qw = new QueryWrapper<>();

		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}

		Page<EvaluationScoringRule> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().like(EvaluationScoringRule::getIndicatorId, indicatorId);
		}

		if (StringUtils.isNotEmpty(scoreRuleName)) {
			qw.lambda().like(EvaluationScoringRule::getScoreRuleName, scoreRuleName);
		}

		if (StringUtils.isNotEmpty(businessPhase)) {
			qw.lambda().like(EvaluationScoringRule::getBusinessPhase, businessPhase);
		}


		IPage<EvaluationScoringRule> evaluationScoringRules = baseMapper.selectPage(page,qw);

		try{
			//设置指标和指标项名称
			List<EvaluationScoringRule> records = evaluationScoringRules.getRecords();
			for (EvaluationScoringRule evaluationScoringRule : records){
				//设置指标名称
				List<String> list = new ArrayList<>();

				List<String> indicatorIds = evaluationScoringRule.getIndicatorIds();
				for (String code : indicatorIds){
					Indicator indicator = indicatorMapper.selectByCode(code);
					if(indicator !=null){
						list.add(indicator.getIndicatorName());
					}
				}
				evaluationScoringRule.setIndicatorNames(list);
				BinSection binSection = binSectionMapper.selectById(evaluationScoringRule.getBusinessPhase());
				if (binSection != null){
					evaluationScoringRule.setBusinessName(binSection.getSectionName());
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
		return evaluationScoringRules;
	}

	@Override
	public boolean saveOrUpdateEvaluationScoringRule(EvaluationScoringRule evaluationScoringRule) {
		int result = 0;

		return false;
	}

	@Override
	@Transactional
	public boolean deleteByIds(List<String> asList) {

		asList.stream().forEach(r ->{
			int i = baseMapper.deleteById(r);
			if (i == 1){
				QueryWrapper<IntervalScoringTable> qw = new QueryWrapper<>();
				if (StringUtils.isNotEmpty(r)) {
					qw.lambda().eq(IntervalScoringTable::getRuleId, r);
				}
				List<IntervalScoringTable> intervalScoringTables = intervalScoringTableMapper.selectList(qw);
				for (IntervalScoringTable intervalScoringTable : intervalScoringTables){
					int id = intervalScoringTableMapper.deleteById(intervalScoringTable.getId());
					if (id != 1){
						throw new CustomException("删除区间评分数据失败！");
					}
				}
			}else {
				throw new CustomException("删除得分规则公式失败！");
			}
		});

		return true;
	}

}
