package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationWarningInfo;
import com.ims.eval.dao.EvaluationWarningInfoMapper;
import com.ims.eval.entity.OrganizationEvaluationRule;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.service.IEvaluationWarningInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-05-19
 */
@Service
public class EvaluationWarningInfoServiceImpl extends ServiceImpl<EvaluationWarningInfoMapper, EvaluationWarningInfo> implements IEvaluationWarningInfoService {


	@Autowired
	private IUserService userService;


	@Override
	public IPage<EvaluationWarningInfo> warningInfoList(Integer pageNum, Integer pageSize, String binSection, String st, String et) {
		QueryWrapper<EvaluationWarningInfo> qw = new QueryWrapper<>();
		//构造分页构造器
		Page<EvaluationWarningInfo> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(EvaluationWarningInfo::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(st)) {
			qw.lambda().ge(EvaluationWarningInfo::getCreateTime, DateUtils.strToDate(st, DateUtils.format1));
		}

		if (StringUtils.isNotEmpty(et)){
			qw.lambda().le(EvaluationWarningInfo::getCreateTime, DateUtils.strToDate(et, DateUtils.format1));
		}
		qw.orderByDesc("create_time");

		IPage<EvaluationWarningInfo> list = baseMapper.selectPage(page, qw);

		return list;
	}


	@Override
	public boolean updateItem(String code, EvaluationWarningInfo warningInfo) {
		if (StringUtils.isNotEmpty(code) && !code.equals("null")){
			MyuserResDTO user = userService.getSysUser(code);
			if (user != null){
				warningInfo.setUserName(user.getName());
				warningInfo.setUserCode(code);
				warningInfo.setDeptId(user.getDeptId());
			}
		}

		return saveOrUpdate(warningInfo);
	}
}
