package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationWarningInfo;
import com.ims.eval.entity.EvaluationWarningRule;
import com.ims.eval.dao.EvaluationWarningRuleMapper;
import com.ims.eval.entity.OrganizationEvaluation;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.service.IEvaluationWarningRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-05-19
 */
@Service
public class EvaluationWarningRuleServiceImpl extends ServiceImpl<EvaluationWarningRuleMapper, EvaluationWarningRule> implements IEvaluationWarningRuleService {

	@Autowired
	private IUserService userService;


	@Override
	public List<EvaluationWarningRule> warningRuleList( String binSection, String ruleName, String type, Boolean enable) {
		QueryWrapper<EvaluationWarningRule> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(binSection)){
			qw.lambda().eq(EvaluationWarningRule::getBinSection, binSection);
		}
		if (StringUtils.isNotEmpty(ruleName)){
			qw.lambda().like(EvaluationWarningRule::getRuleName, ruleName);
		}
		if (StringUtils.isNotEmpty(type)){
			qw.lambda().eq(EvaluationWarningRule::getType, type);
		}
		if (enable != null){
			qw.lambda().eq(EvaluationWarningRule::getEnable, enable);
		}
		List<EvaluationWarningRule> list = baseMapper.selectList(qw);
		return list;
	}


	@Override
	public IPage<EvaluationWarningRule> warningRuleList(Integer pageNum, Integer pageSize, String binSection, String ruleName, String type, Boolean enable) {
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluationWarningRule> page = new Page<>(pageNum, pageSize);
		IPage<EvaluationWarningRule> list = baseMapper.selectWarningRule(page, binSection, ruleName, type, enable);
		return list;
	}

	@Override
	public boolean saveOrUpdate(String code, EvaluationWarningRule warningRule) {
		if (StringUtils.isNotEmpty(code) && !code.equals("null")){
			MyuserResDTO user = userService.getSysUser(code);
			if (user != null){
				if (StringUtils.isNotEmpty(warningRule.getId())){
					warningRule.setUpdateBy(user.getName());
					warningRule.setUpdateTime(new Date());
				} else {
					warningRule.setCreateBy(user.getName());
					warningRule.setCreateTime(new Date());
				}
			}
		}
		return saveOrUpdate(warningRule);
	}
}
