package com.ims.eval.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ims.common.utils.FormulaUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.CalculateIndicatorItemInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.dao.OrganizationEvaluationInfoMapper;
import com.ims.eval.entity.dto.response.OrganizationEvaluationInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ims.eval.util.ExcelUtil.readExcel;

/**
 * <p>
 * 考评指标明细

 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-16
 */
@Service
@Log4j2
public class OrganizationEvaluationInfoServiceImpl extends ServiceImpl<OrganizationEvaluationInfoMapper, OrganizationEvaluationInfo> implements IOrganizationEvaluationInfoService {

	@Autowired
	private ICalculateIndicatorItemInfoService iCalculateIndicatorItemInfoService;


	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Autowired
	private IUserService userService;

	@Autowired
	private IEvaluationScoreCountService iEvaluatioinScoreCountService;


	@Autowired
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	@Override
	public Map finishValueList(String organizationEvaluationId, String dept, HttpServletRequest request) {

		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId(organizationEvaluationId,dept,"","");//默认获取指定的怎目标
		list.stream().forEach(i->{
			i.setUpdateMark(true);
		});
		Map<String  ,List<OrganizationEvaluationInfoResDTO>> map  = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove("是"));
		map.put("nonQuantifiedList", map.remove("否"));
		return map;
	}

	@Transactional
	@Override
	public boolean saveBatchDto(List<OrganizationEvaluationInfo> organizationEvaluationInfoList) throws Exception {
		for (OrganizationEvaluationInfo entity : organizationEvaluationInfoList) {

			if (null != entity && null != entity.getId() && !"".equals(entity.getId().trim())) {
				OrganizationEvaluationInfo info = baseMapper.selectById(entity.getId());

				info.setQuantifiedValue(entity.getQuantifiedValue());//量化值
				if(entity.getIsQuantified().equals("否") && "JKF".equals(info.getOptionCode())){//加扣分
					if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
						info.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
					}
				}

				if(entity.getIsQuantified().equals("否") && "JCF".equals(info.getOptionCode())){//基础分
					if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
						info.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
					}
				}

				info.setNonQuantifiedValue(entity.getNonQuantifiedValue());//非量化值
				info.setUpdateTime(new Date());//更新时间
				info.setUpdateBy(entity.getUpdateBy());//更新者
				info.setRemark(entity.getRemark());//备注
				info.setState(entity.getState());//初始状态
				boolean b = super.saveOrUpdate(info);
				if(!b){
					throw new CustomException("更新失败");
				}
			}else {
				entity.setCreateTime(new Date());

				if (StringUtils.isEmpty(entity.getIndicatorId())) {
					throw new CustomException("保存失败");
				}
				List<IndicatorDictionary> dictionaryList =  indicatorDictionaryService.list("",entity.getIndicatorId(),"","");
				for(IndicatorDictionary d : dictionaryList){

					OrganizationEvaluationInfo addinfo = new OrganizationEvaluationInfo();
					addinfo.setOrganizationEvaluationId(entity.getOrganizationEvaluationId());
					addinfo.setOrganizationEvaluationRuleId(entity.getOrganizationEvaluationRuleId());
					addinfo.setIndicatorId(entity.getIndicatorId());
					addinfo.setIndicatorDictionaryId(d.getId());
					addinfo.setOptionCode(d.getOptionCode());
					addinfo.setChildCode(d.getChildCode());
					addinfo.setIsQuantified(entity.getIsQuantified());
					addinfo.setQuantifiedValue(entity.getQuantifiedValue());

					if(entity.getIsQuantified().equals("否") && "JKF".equals(d.getOptionCode())){//加扣分
						if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
							addinfo.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
						}
					}

					if(entity.getIsQuantified().equals("否") && "JCF".equals(d.getOptionCode())){//基础分
						if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
							addinfo.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
						}
					}
					addinfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());
					addinfo.setState("-1");//初始状态
					addinfo.setCreateTime(new Date());

					boolean b = super.saveOrUpdate(addinfo);
					if(!b){
						throw new CustomException("保存失败");
					}
				}

			}

		}
//废弃方法
//		OrganizationEvaluationInfo organizationEvaluationInfo = baseMapper.selectById(organizationEvaluationInfoList.get(0).getId());
//		//查询本次考评所有考评指标明细
//		List<OrganizationEvaluationInfo> organizationEvaluationInfos = baseMapper.selectByEvaluationInfoId(organizationEvaluationInfo.getOrganizationEvaluationId());
//
//		//调用需要计算指标项的方法
//		boolean calculate = calculateIdicatorItem(organizationEvaluationInfos);
//
//		//调用计算存储最大值，最小值的方法
//		if (calculate){
//			iCalculateIndicatorItemInfoService.saveIndicatorInfo(organizationEvaluationInfos);
//		}


		return true;
	}






	@Override
	public List<OrganizationEvaluationInfoResDTO> finishValueList(String organizationEvaluationId, String dept, String organizationShortName, String indicatorName, String binSection,String isQuantified, HttpServletRequest request) {
		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId2(organizationEvaluationId,dept,organizationShortName,indicatorName,binSection,isQuantified,"","");//默认获取指定的怎目标
		list.stream().forEach(i->{
			i.setUpdateMark(true);
		});
		return list;
	}

	@Override
	public boolean calculationByEvaluationId(String id, String indicatorId,String organizationEvaluationId,String optionCode) {
		try {
			List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectByIdAndEvaluationInfoId(id,indicatorId,organizationEvaluationId,"");
			//根据板块分组
			Map<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropList = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getBinSection));

			for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropEntry : binSectionGropList.entrySet()){
				if (binSectionGropEntry.getValue().size() > 0){
					//按照指标分组
					Map<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropList = binSectionGropEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIndicatorId));


					//计算前置属性（差值、利润贡献、单位千瓦利润）的最大最小值
					for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropListEntry : indicatorGropList.entrySet()){

						if (indicatorGropListEntry.getValue().size() > 0){
							//按照公司规则分组
							Map<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropList = indicatorGropListEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationEvaluationRuleId));

							//遍历公司指标
							for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropListEntry : ruleIdGropList.entrySet()){
								if (ruleIdGropListEntry.getValue().size() > 0) {

									List<OrganizationEvaluationInfoResDTO> indicator = ruleIdGropListEntry.getValue();
									Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = indicator.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));

									double totalScore = 0.00;
									for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()){
										for (OrganizationEvaluationInfoResDTO dto : childCodeGropListEntry.getValue()) {

											if ((!dto.getOptionCode().equals("CZ") && !dto.getOptionCode().equals("LRGXL") && !dto.getOptionCode().equals("DWQWLR")) || StringUtils.isEmpty(dto.getFormula())) {
												continue;//当前属性没有配置公式跳过
											}

											for (OrganizationEvaluationInfoResDTO f : childCodeGropListEntry.getValue()) {
												dto.setFormula(dto.getFormula().replace("[" + f.getOptionCode() + "]", f.getQuantifiedValue() + ""));
											}
											double score = FormulaUtils.calculateFormula(dto.getFormula());
											if (StringUtils.inStringIgnoreCase("DF", dto.getOptionCode())) {
												totalScore = totalScore + score;
											}
											log.info(dto.getOrganizationShortName() + "|" + dto.getChildName()+"|"+dto.getOptionCode() + "----------" + dto.getFormula() + "=" + score);
											OrganizationEvaluationInfo info = baseMapper.selectById(dto.getId());
											//将计算结果保存
											info.setQuantifiedValue(score);
											dto.setQuantifiedValue(score);
											this.saveOrUpdate(info);
										}
								}
								}
							}
						}
						//计算差值等的最大值最小值
						iCalculateIndicatorItemInfoService.saveIndicatorInfoDTO(indicatorGropListEntry.getValue());
					}


//					//计算前置属性（差值、利润贡献、单位千瓦利润）的最大最小值
					for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropListEntry : indicatorGropList.entrySet()){
						for (OrganizationEvaluationInfoResDTO d : indicatorGropListEntry.getValue()){
							System.out.println(d.getOrganizationShortName()+"|"+d.getIndicatorName()+"|"+d.getOptionName()+"|"+d.getQuantifiedValue());
						}

						if (indicatorGropListEntry.getValue().size() > 0){
							//按照公司规则分组
							Map<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropList = indicatorGropListEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationEvaluationRuleId));


							//遍历公司指标
							for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropListEntry : ruleIdGropList.entrySet()){
								if (ruleIdGropListEntry.getValue().size() > 0){

									List<OrganizationEvaluationInfoResDTO> indicator = ruleIdGropListEntry.getValue();
									Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = indicator.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));
									double totalScore = 0.00;
									EvaluationScoreCount scoreCount = new EvaluationScoreCount();
									double standardScore = 0;
									for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()){

										List<CalculateIndicatorItemInfo> calculateIndicatorItemInfo = calculateIndicatorItemInfoMapper.selectList("","","","","",childCodeGropListEntry.getValue().get(0).getOrganizationEvaluationId(),childCodeGropListEntry.getValue().get(0).getChildCode());
										Map<String ,Double>  map = new HashMap<>();
										for (CalculateIndicatorItemInfo itemInfo :calculateIndicatorItemInfo){
											map.put(itemInfo.getOptionCode(),itemInfo.getQuantifiedValue());
										}
										for (OrganizationEvaluationInfoResDTO dto: childCodeGropListEntry.getValue()){
											if ((dto.getOptionCode().endsWith("MAX") ||dto.getOptionCode().endsWith("MIN")
												||dto.getOptionCode().equals("CZ") || dto.getOptionCode().equals("LRGXL")
												|| dto.getOptionCode().equals("DWQWLR")) || StringUtils.isEmpty(dto.getFormula())){
												continue;//当前属性没有配置公式跳过或者不需要参与此次计算的属性
											}
											scoreCount.setOrganizationEvaluationId(dto.getOrganizationEvaluationId());
											scoreCount.setOrganizationEvaluationRuleId(dto.getOrganizationEvaluationRuleId());
											scoreCount.setIndicatorId(dto.getIndicatorId());
											scoreCount.setStageCode(dto.getStageCode());

											for (OrganizationEvaluationInfoResDTO f : indicator){
												if(f.getOptionCode().endsWith("MIN") || f.getOptionCode().endsWith("MAX")){
													f.setQuantifiedValue(map.get(f.getOptionCode()));
												}
												dto.setFormula(dto.getFormula().replace("["+f.getOptionCode()+"]", f.getQuantifiedValue() + ""));
											}
											log.info("计算公式为："+dto.getFormula());
											double score = FormulaUtils.calculateFormula(dto.getFormula());
											if(dto.getOptionCode().equals("DF")||dto.getOptionCode().equals("DF2")
												||dto.getOptionCode().equals("JHDF")){
												standardScore = dto.getStandardScore();
												totalScore = totalScore+score;
												//保存加减分原因
												if("YY".equals(dto.getOptionCode())){
													scoreCount.setRemark(dto.getNonQuantifiedValue());
												}else if(!"ZDZX".equals(dto.getStageCode()) && !"GLSX".equals(dto.getStageCode())){
													scoreCount.setRemark(dto.getIndicatorName());
												}
											}
											log.info(dto.getOrganizationShortName()+"|"+dto.getIndicatorName()+"|"+dto.getOptionCode()+"----------"+dto.getFormula()+"="+score);
											OrganizationEvaluationInfo info  = baseMapper.selectById(dto.getId());
											//将计算结果保存
											info.setQuantifiedValue(score);
											dto.setQuantifiedValue(score);
											this.saveOrUpdate(info);
										}
									}

									log.info("综合得分："+totalScore);
									if(null==scoreCount.getOrganizationEvaluationId()|| null== scoreCount.getOrganizationEvaluationRuleId() || null == scoreCount.getIndicatorId()){
											continue;
									}
									if(standardScore>0 && totalScore != standardScore){
										scoreCount.setStandard(totalScore>standardScore?"1":"-1");
									}else {
										scoreCount.setStandard("0");
									}
									scoreCount.setScore(totalScore);
									scoreCount.setObversionScore(totalScore);
									scoreCount.setIsQuantified("1");

									List<EvaluationScoreCount>  listcount = iEvaluatioinScoreCountService.getEvaluationScoreCountList(scoreCount.getOrganizationEvaluationId(),scoreCount.getOrganizationEvaluationRuleId(),scoreCount.getIndicatorId());
									if(null  != listcount && listcount.size() >0){
										List<String> ids = listcount.stream().map(EvaluationScoreCount::getId).collect(Collectors.toList());
										boolean cod = iEvaluatioinScoreCountService.removeByIds(ids);
									}
									iEvaluatioinScoreCountService.save(scoreCount);
								}
							}
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	@Override
	public List<Map> getEvaluationIndicatorList(String organizationEvaluationId, String binSection, String binStage, HttpServletRequest request) {
		List<Map> indicatorMap = baseMapper.selectEvaluationIndicatorList(organizationEvaluationId,binSection,binStage);
		return indicatorMap;
	}

	@Override
	public Map getEvaluationInfoList(String organizationEvaluationId, String indicatorId, String binSection, String binStage, HttpServletRequest request) {

		Map data = new HashMap();
		boolean titlemark = true;//标记生成标题
		List<OrganizationEvaluationInfoResDTO> infoResDTOList = baseMapper.selectEvaluationInfoList(organizationEvaluationId, indicatorId,binSection,binStage);
		List<IndicatorDictionary> dictionaryList = indicatorDictionaryService.list("", indicatorId, "", "");

		//根据组织（公司）id分组
		Map<String, List<OrganizationEvaluationInfoResDTO>> groupedData = infoResDTOList.stream()
			.collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationId));
		List<Map> mapList = new ArrayList<>();
		//遍历按照公司分组后的数据
		for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> entry : groupedData.entrySet()) {

			Map indicatormap = new HashMap();
			List<OrganizationEvaluationInfoResDTO> dtoList = entry.getValue();

			Map<String, OrganizationEvaluationInfoResDTO> resultMap = dtoList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue)); // 解决键重复的情况
			boolean mark = true;//标记给公司名赋值

			//将子指标分组
			Map<String, List<IndicatorDictionary>> groupedChildCode = dictionaryList.stream()
				.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));


			if (titlemark) {
				Map title = new LinkedHashMap();
				for (Map.Entry<String, List<IndicatorDictionary>> childCodeEntry : groupedChildCode.entrySet()) {


					List<Map> titleArray = new ArrayList<>();
					for (IndicatorDictionary d : childCodeEntry.getValue()) {
						Map titlemap = new LinkedHashMap();
						if (!d.getIsShow()) {
							continue;
						}
						titlemap.put("key", d.getOptionName());//名称
						titlemap.put("code",d.getChildCode()+"_"+d.getOptionCode());//名称
						titleArray.add(titlemap);

					}
					title.put(childCodeEntry.getKey().split(",")[1], titleArray);

				}
				titlemark = false;
				data.put("title", title);
			}

			for (IndicatorDictionary d : dictionaryList) {
				if (!d.getIsShow()) {
					continue;
				}

				if (mark) {
					indicatormap.put("organizationShortName", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationShortName());//公司名
					indicatormap.put("organizationId", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationId());//公司id
					mark = false;
				}

				System.out.println(resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//指标属性
				indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue());
				//指标id
				indicatormap.put("ID_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//是否量化
				indicatormap.put("IS_LH_" +d.getChildCode() + "_" + d.getOptionCode(),resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getIsQuantified2());
			}
			mapList.add(indicatormap);
		}

		data.put("value", mapList);
		return data;
	}

	@Override
	public byte[] downloadExcel(String organizationEvaluationId, String indicatorId, String binSection, String binStage,HttpServletRequest request) throws Exception{


		Map<String ,List<Map<String,String>>> title = new LinkedHashMap();//标题
		List<Map<String,String>> mapList = new ArrayList<>();//数据list
		boolean titlemark = true;//标记生成标题
		List<OrganizationEvaluationInfoResDTO> infoResDTOList = baseMapper.selectEvaluationInfoList(organizationEvaluationId, indicatorId,binSection,binStage);
		List<IndicatorDictionary> dictionaryList = indicatorDictionaryService.list("", indicatorId, "", "");

		//根据组织（公司）id分组
		Map<String, List<OrganizationEvaluationInfoResDTO>> groupedData = infoResDTOList.stream()
			.collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationId));

		//遍历按照公司分组后的数据
		for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> entry : groupedData.entrySet()) {

			Map indicatormap = new LinkedHashMap();
			List<OrganizationEvaluationInfoResDTO> dtoList = entry.getValue();

			Map<String, OrganizationEvaluationInfoResDTO> resultMap = dtoList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue)); // 解决键重复的情况
			boolean mark = true;//标记给公司名赋值

			//将子指标分组
			Map<String, List<IndicatorDictionary>> groupedChildCode = dictionaryList.stream()
				.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));

			if (titlemark) {
				for (Map.Entry<String, List<IndicatorDictionary>> childCodeEntry : groupedChildCode.entrySet()) {


					List<Map<String,String>> titleArray = new ArrayList<>();
					Map titlemap = new LinkedHashMap();
					for (IndicatorDictionary d : childCodeEntry.getValue()) {
						if (!d.getIsShow()) {
							continue;
						}
						titlemap.put(d.getChildCode()+"_"+d.getOptionCode(), d.getOptionName());//名称
						titlemap.put("ID_" +d.getChildCode()+"_"+d.getOptionCode(), "ID隐藏");//id
						titlemap.put("IS_LH_" +d.getChildCode()+"_"+d.getOptionCode(), "是否量化隐藏");//是否量化
					}
					titleArray.add(titlemap);
					title.put(childCodeEntry.getKey().split(",")[1], titleArray);

				}
				titlemark = false;
			}



			for (IndicatorDictionary d : dictionaryList) {
				if (!d.getIsShow()) {
					continue;
				}
				if (mark) {
					indicatormap.put("organizationShortName", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationShortName());//公司名
					mark = false;
				}

				System.out.println(resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//指标属性
				indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue());
				//指标id
				indicatormap.put("ID_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//是否量化
				indicatormap.put("IS_LH_" +d.getChildCode() + "_" + d.getOptionCode(),resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getIsQuantified2().toString());
			}
			mapList.add(indicatormap);
		}


		// 创建Workbook对象
		Workbook workbook = new XSSFWorkbook();

		// 创建工作表
		Sheet sheet = workbook.createSheet(infoResDTOList.get(0).getIndicatorName());

		// 创建单元格样式
		CellStyle cellStyle = workbook.createCellStyle();
		cellStyle.setAlignment(HorizontalAlignment.CENTER);
		cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);

		// 设置边框样式
		cellStyle.setBorderTop(BorderStyle.THIN);
		cellStyle.setBorderBottom(BorderStyle.THIN);
		cellStyle.setBorderLeft(BorderStyle.THIN);
		cellStyle.setBorderRight(BorderStyle.THIN);

		// 创建1、2行作为标题行
		Row headerRow0 = sheet.createRow(0);
		Row headerRow1 = sheet.createRow(1);
		Row headerRow2 = sheet.createRow(2);
		Row headerRow3 = sheet.createRow(3);


		// 设置首行的单元格值
		Cell header0Cell0 = headerRow0.createCell(0);
		header0Cell0.setCellValue(infoResDTOList.get(0).getIndicatorName());


		Cell headerCell0 = headerRow1.createCell(0);
		headerCell0.setCellValue("序号");
		headerCell0.setCellStyle(cellStyle);
		headerCell0.setCellStyle(cellStyle);
		sheet.addMergedRegion(new CellRangeAddress(1, 2, 0, 0));

		Cell headerCell1 = headerRow1.createCell(1);
		headerCell1.setCellValue("单位名称");
		headerCell1.setCellStyle(cellStyle);
		headerCell1.setCellStyle(cellStyle);
		sheet.addMergedRegion(new CellRangeAddress(1, 2, 1, 1));


        Integer colsize = 1;
		Integer optioncols = 2;
		for (String  key : title.keySet()) {//存在多个标题
			List<Map<String,String>> titleArray2 = title.get(key);
			Cell headerCellcolsize = headerRow1.createCell(colsize+1);
			headerCellcolsize.setCellValue(key);
			headerCellcolsize.setCellStyle(cellStyle);
			headerCellcolsize.setCellStyle(cellStyle);

			System.out.println("key="+key);

			for (Map<String,String>  key2 : titleArray2) {
				sheet.addMergedRegion(new CellRangeAddress(1, 1, colsize+1, colsize + key2.size()));
				colsize = colsize + key2.size();
				for (String  key3 : key2.keySet()) {

					Cell header3Cell2 = headerRow3.createCell(optioncols);
					header3Cell2.setCellValue(key3);
					header3Cell2.setCellStyle(cellStyle);

					Cell header2Cell2 = headerRow2.createCell(optioncols);
					header2Cell2.setCellValue(key2.get(key3));
					header2Cell2.setCellStyle(cellStyle);

					// 隐藏列
					if(key3.startsWith("ID_") || key3.startsWith("IS_LH_")){
						sheet.setColumnHidden(optioncols, true);
					}
					optioncols++;
					System.out.println(key2.get(key3)+"="+key3);
				}
			}
		}

		Integer rowdate = 4;
		for (Map<String,String >  datamaps  : mapList) {
			Integer colsdata = 1;
			Row headerRowdate = sheet.createRow(rowdate);
			Cell dataCell0 = headerRowdate.createCell(0);
			dataCell0.setCellValue(rowdate-3);
			dataCell0.setCellStyle(cellStyle);
			for (String datakey: datamaps.keySet()) {

				Cell dataCell1 = headerRowdate.createCell(colsdata);
				dataCell1.setCellValue(datamaps.get(datakey).toString());
				dataCell1.setCellStyle(cellStyle);
				colsdata++;
			}
			rowdate++;
		}

		//隐藏行
		sheet.getRow(3).setZeroHeight(true);
		sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, colsize));
		header0Cell0.setCellStyle(cellStyle);

		// 将Workbook写入字节数组输出流
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		workbook.write(outputStream);

		// 将字节数组作为响应体返回，并设置响应头
		byte[] excelBytes = outputStream.toByteArray();
		return excelBytes;
	}

	@Override
	public boolean importExcel(MultipartFile file) throws IOException {

		List<Map<String, Object>> dataList = new ArrayList<>();
		try {
			//获取原始的文件名
			String originalFilename = file.getOriginalFilename();
			//获取文件类型
			String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
			//获取输入流
			InputStream is = file.getInputStream();
			Workbook workbook = readExcel(fileType, is);

			Sheet sheet = workbook.getSheetAt(0);

			// 获取总列数和起始行索引
			int totalColumns = sheet.getRow(3).getLastCellNum();
			int startingRowIndex = 4;

			Row headerRow = sheet.getRow(3);


			// 遍历每一行数据
			for (int i = startingRowIndex; i <= sheet.getLastRowNum(); i++) {
				Row row = sheet.getRow(i);
				Map<String, Object> dataMap = new LinkedHashMap<>();

				// 遍历每一列数据
				for (int j = 1; j < totalColumns; j++) {
					Cell headerCell = headerRow.getCell(j);
					Cell cell = row.getCell(j);

					String columnName = headerCell.getStringCellValue();
					String cellValue = "";

					// 根据单元格类型读取对应内容
					if (cell != null) {
						switch (cell.getCellType()) {
							case HSSFCell.CELL_TYPE_STRING:
								cellValue = cell.getStringCellValue();
								break;
							case HSSFCell.CELL_TYPE_NUMERIC:
								cellValue = String.valueOf(cell.getNumericCellValue());
								break;
							case HSSFCell.CELL_TYPE_BOOLEAN:
								cellValue = String.valueOf(cell.getBooleanCellValue());
								break;
							case HSSFCell.CELL_TYPE_BLANK:
								cellValue = "";
								break;
						}
					}

					dataMap.put(columnName, cellValue);
				}

				dataList.add(dataMap);
			}
			workbook.close();
			for(Map<String, Object> map : dataList){

				Map<String,Object > idMap = new HashMap();
				Map valueMap =  new HashMap();
				for (String key : map.keySet()) {
					Object value = map.get(key);
					if(key.startsWith("ID_")){
						idMap.put(key,value);
					}else {
						valueMap.put(key,value);
					}
				}

				List<OrganizationEvaluationInfo> evaluationInfos =new ArrayList<>();
				for (Map.Entry<String,Object > entry : idMap.entrySet()) {
					OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
					info.setId(entry.getValue().toString());
					boolean quantified = Boolean.valueOf(valueMap.get(entry.getKey().replace("ID_","IS_LH_")).toString());
					if(quantified){
						info.setQuantifiedValue(Double.valueOf(valueMap.get(entry.getKey().replace("ID_","")).toString()));
					}
					info.setNonQuantifiedValue(valueMap.get(entry.getKey().replace("ID_","")).toString());
					evaluationInfos.add(info);
				}
				this.saveOrUpdateBatch(evaluationInfos);

			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;

	}

	@Transactional
	@Override
	public boolean updateEvaluationInfo(List<JSONObject> jsonObjects) {
		for (JSONObject json :jsonObjects){

			Map<String,Object > idMap = new HashMap();
			Map valueMap =  new HashMap();
			for (String key : json.keySet()) {
				Object value = json.get(key);
				if(key.startsWith("ID_")){
					idMap.put(key,value);
				}else {
					valueMap.put(key,value);
				}
			}

			List<OrganizationEvaluationInfo> evaluationInfos =new ArrayList<>();
			for (Map.Entry<String,Object > entry : idMap.entrySet()) {
				OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
				info.setId(entry.getValue().toString());
				boolean quantified = Boolean.valueOf(valueMap.get(entry.getKey().replace("ID_","IS_LH_")).toString());
				if(quantified){
					info.setQuantifiedValue(Double.valueOf(valueMap.get(entry.getKey().replace("ID_","")).toString()));
				}
				info.setNonQuantifiedValue(valueMap.get(entry.getKey().replace("ID_","")).toString());
				evaluationInfos.add(info);
			}
			this.saveOrUpdateBatch(evaluationInfos);

		}
		return true;
	}

	//todo  后续添加需要计算的指标项
	public boolean calculateIdicatorItem(List<OrganizationEvaluationInfo> organizationEvaluationInfos){


		//根据考评规则分组
		Map<String, List<OrganizationEvaluationInfo>> collect1 = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry1 : collect1.entrySet()){
			//根据指标id分组
			Map<String, List<OrganizationEvaluationInfo>> collect = entry1.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
			for(Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()){
				//过滤差值考评指标明细
				List<OrganizationEvaluationInfo> cz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("CZ")).collect(Collectors.toList());
				if (cz.size() == 1){
					List<OrganizationEvaluationInfo> wcz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("WCZ")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> value = entry.getValue().stream().filter(r -> r.getOptionCode().equals("JHZ") || r.getOptionCode().equals("DBZ") || r.getOptionCode().equals("MBZ")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (wcz.size() == 1 && value.size() == 1){
						double wczValue = wcz.get(0).getQuantifiedValue();
						double dbzvalue = value.get(0).getQuantifiedValue();
						cz.get(0).setQuantifiedValue(wczValue - dbzvalue);
						cz.get(0).setCreateTime(new Date());
						boolean b = super.saveOrUpdate(cz.get(0));
						if(!b){
							throw new CustomException("更新失败");
						}
					}
				}

				//过滤单位千瓦利润考评指标明细
				List<OrganizationEvaluationInfo> dwqwlr = entry.getValue().stream().filter(r -> r.getOptionCode().equals("DWQWLR")).collect(Collectors.toList());
				if (dwqwlr.size() == 1){
					List<OrganizationEvaluationInfo> lrgxl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("LRGXL")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> zjrl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("ZJRL")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (lrgxl.size() == 1 && zjrl.size() == 1){
						double wczValue = lrgxl.get(0).getQuantifiedValue();
						double dbzvalue = zjrl.get(0).getQuantifiedValue();
						if (dbzvalue == 0 || dbzvalue == 0.0){
							dwqwlr.get(0).setQuantifiedValue(0);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}else{
							//double保留两位小数
							double v = new BigDecimal(wczValue / dbzvalue).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
							dwqwlr.get(0).setQuantifiedValue(v);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}
					}
				}

			}
		}
		return true;
	}



}
