package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.*;
import com.ims.eval.dao.OrganizationEvaluationRuleMapper;
import com.ims.eval.entity.dto.request.OrganizationEvaluationRuleDTO;
import com.ims.eval.service.IDeptResponsibilityService;
import com.ims.eval.service.IEvaluateRuleService;
import com.ims.eval.service.IOrganizationEvaluationRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.service.IOrganizationEvaluationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 单位/部门考评配置 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-01
 */
@Service
public class OrganizationEvaluationRuleServiceImpl extends ServiceImpl<OrganizationEvaluationRuleMapper, OrganizationEvaluationRule> implements IOrganizationEvaluationRuleService {


	@Autowired
	private IEvaluateRuleService evaluateRuleService;

	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;


	@Autowired
	private IDeptResponsibilityService deptResponsibilityService;


	@Override
	public IPage<OrganizationEvaluationRule> list(Integer pageNum, Integer pageSize, String id, String organizationName, String organizationId, String organizationType, String binSection, String binStage, String evaluationCycle, String year) {
		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
		//构造分页构造器
		Page<OrganizationEvaluationRule> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(OrganizationEvaluationRule::getId, id);
		}

		if (StringUtils.isNotEmpty(organizationName)) {
			qw.lambda().like(OrganizationEvaluationRule::getOrganizationName, organizationName);
		}

		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().like(OrganizationEvaluationRule::getOrganizationId, organizationId);
		}

		if (StringUtils.isNotEmpty(organizationType)) {
			qw.lambda().eq(OrganizationEvaluationRule::getOrganizationType, organizationType);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(OrganizationEvaluationRule::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(OrganizationEvaluationRule::getBinStage, binStage);
		}

		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, evaluationCycle);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(OrganizationEvaluationRule::getYear, year);
		}

		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, false);


		qw.lambda().orderByAsc(OrganizationEvaluationRule::getOrderNum);

		IPage<OrganizationEvaluationRule> list = baseMapper.selectPage(page, qw);

		list.getRecords().stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
		});


		return list;
	}

	@Override
	public List<OrganizationEvaluationRule> listAll(String id, String organizationName, String organizationId, String organizationType, String binSection, String binStage, String evaluationCycle, String year) {


		List<OrganizationEvaluationRule> list = baseMapper.selectListAll(id, organizationName, organizationId, organizationType, binSection, binStage, evaluationCycle,year);

		list().stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));

			StringBuilder binStageName =  new StringBuilder();
			for (String stage : l.getBinStage().split(",")){
				binStageName.append(null == CacheContext.ddNameMap.get(stage) ? "" :CacheContext.ddNameMap.get(stage)).append(",");
			}

			l.setBinStageName(binStageName.toString().length()>0?binStageName.toString().substring(0,binStageName.toString().length()-1):"");

			StringBuilder ruleName = new StringBuilder();
			for (String ruleId :l.getEvaluateRuleId().split(",")){
				EvaluateRule rulebyid = evaluateRuleService.getById(ruleId);
				if(null != rulebyid){
					ruleName.append(rulebyid.getRuleName()).append(",");
				}

			}
			l.setEvaluateRuleName(ruleName.toString().length()>0?ruleName.toString().substring(0,ruleName.toString().length()-1):"");

		});

		return list;
	}

	@Override
	public IPage<OrganizationEvaluationRule> list2(Integer pageNum, Integer pageSize, String id, String organizationName, String organizationId, String organizationType, String binSection, String binStage, String evaluationCycle, String year) {
		Page<OrganizationEvaluationRule> page = new Page<>(pageNum, pageSize);
		IPage<OrganizationEvaluationRule>  list = baseMapper.list(page, id, organizationName, organizationId, organizationType, binSection, binStage, evaluationCycle,year);
		list.getRecords().stream().forEach(l->{

			StringBuilder binStageName =  new StringBuilder();
			for (String stage : l.getBinStage().split(",")){
				binStageName.append(null == CacheContext.bseIdMap.get(stage) ? "" :CacheContext.bseIdMap.get(stage)).append(",");
			}

			l.setBinStageName(binStageName.toString().length()>0?binStageName.toString().substring(0,binStageName.toString().length()-1):"");

			StringBuilder ruleName = new StringBuilder();
			for (String ruleId :l.getEvaluateRuleId().split(",")){
				EvaluateRule rulebyid = evaluateRuleService.getById(ruleId);
				if(null != rulebyid){
					ruleName.append(rulebyid.getRuleName()).append(",");
				}

			}
			l.setEvaluateRuleName(ruleName.toString().length()>0?ruleName.toString().substring(0,ruleName.toString().length()-1):"");

		});
		return list;
	}


	@Override
	public List<OrganizationEvaluationRule> listByIsCheck(String organizationType,String evaluationCycle, Boolean isCheck, Boolean delFlag) {

		if (StringUtils.isEmpty(organizationType) || StringUtils.isEmpty(evaluationCycle)) {
			throw new CustomException("获取考评配置时，考评类型为空");
		}

		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();


		if (StringUtils.isNotEmpty(organizationType)) {
			qw.lambda().eq(OrganizationEvaluationRule::getOrganizationType, organizationType);
		}

		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, evaluationCycle);
		}

		if (null != isCheck) {
			qw.lambda().eq(OrganizationEvaluationRule::getIsCheck, isCheck);
		}

		if (null != delFlag) {
			qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, delFlag);
		}

		List<OrganizationEvaluationRule> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public List<OrganizationEvaluationRule> getOrganizationRuleId(String evalOrRespId, String type) {

		List<String> ids = new ArrayList<>();
		if("mb".equals(type)){
			DeptResponsibility deptResponsibility  = deptResponsibilityService.getById(evalOrRespId);
			if(null == deptResponsibility){
				throw new CustomException("考评记录为空");
			}
			ids = Arrays.asList(deptResponsibility.getOrganizationEvaluationRuleId().split(","));
		}
		if("kp".equals(type)){
			OrganizationEvaluation  organizationEvaluation = organizationEvaluationService.getById(evalOrRespId);
			if(null == organizationEvaluation){
				throw new CustomException("考评记录为空");
			}
			ids = Arrays.asList(organizationEvaluation.getOrganizationEvaluationRuleId().split(","));
		}


		if(ids.size()<=0){
			throw new CustomException("考评记录重配置为空");
		}
//		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
//		qw.lambda().in(OrganizationEvaluationRule::getId, ids);
		List<OrganizationEvaluationRule> list = baseMapper.selectOrganizationRuleId(ids);

		return list;
	}

	@Override
	public OrganizationEvaluationRule getByIdIsCheck(String id,Boolean isCheck,Boolean delFlag) {

		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(OrganizationEvaluationRule::getId, id);
		}
		qw.lambda().eq(OrganizationEvaluationRule::getIsCheck, isCheck);
		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, delFlag);
		List<OrganizationEvaluationRule> list  = baseMapper.selectList(qw);
		if(null != list && list.size()>0){
			return list.get(0);
		}
		return null;
	}

	@Override
	public List<OrganizationEvaluationRule> getOrganizationEvaluationRuleByIds(List<String> ids, String binSection) {

		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(ids)) {
			qw.lambda().in(OrganizationEvaluationRule::getId, ids);
		}

		if (StringUtils.isNotEmpty(ids)) {
			qw.lambda().like(OrganizationEvaluationRule::getBinSection, binSection);
		}

		List<OrganizationEvaluationRule> list  = baseMapper.selectList(qw);

		return list;
	}

	@Override
	public List<OrganizationEvaluationRule> getOrganizationEvaluationRuleByYearAndCycle(String organizationType,String evaluationCycle, String year) {

		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(organizationType)) {
			qw.lambda().eq(OrganizationEvaluationRule::getOrganizationType, organizationType);
		}
		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, evaluationCycle);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(OrganizationEvaluationRule::getYear, year);
		}


		//没有删除标记，并且是需要考核的单位
		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, false);
		qw.lambda().eq(OrganizationEvaluationRule::getIsCheck, true);

		List<OrganizationEvaluationRule> list  = baseMapper.selectList(qw);

		if(null != list && list.size()>0){
			return list;
		}
		return null;
	}


	@Transactional
	@Override
	public boolean saveOrUpdate(OrganizationEvaluationRule entity) {
		//如果是存在主键  则修改
		if (null != entity && (null != entity.getId() && !"".equals(entity.getId().trim()))) {
			OrganizationEvaluationRule update = baseMapper.selectById(entity.getId());
			update.setDelFlag(true);
			update.setUpdateTime(new Date());
			baseMapper.updateById(update);
		}

		List<String> evaluateRuleIds = Arrays.asList(entity.getEvaluateRuleId().split(","));
		List<EvaluateRule> evaluateRuleList = evaluateRuleService.listAll(evaluateRuleIds);
		List<String> binSections = evaluateRuleList.stream().map(EvaluateRule::getBinSection).collect(Collectors.toList());
		List<String> binStages = evaluateRuleList.stream().map(EvaluateRule::getBinStage).collect(Collectors.toList());

		if (StringUtils.isEmpty(entity.getBinSection())) {
			entity.setBinSection(String.join(",", binSections));
		}
		if (StringUtils.isEmpty(entity.getBinStage())) {
			entity.setBinStage(String.join(",", binStages));
		}

		//型增时判断当前组织是否存在同期考评配置
		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
		qw.lambda().eq(OrganizationEvaluationRule::getOrganizationId, entity.getOrganizationId());
		qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, entity.getEvaluationCycle());
//		qw.lambda().eq(OrganizationEvaluationRule::getBinSection, entity.getBinSection());

		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, false);
		List<OrganizationEvaluationRule> list = baseMapper.selectList(qw);
		if (null != list && list.size() > 0) {
			throw new CustomException("已存在同期考评配置");
		}

		//判断排序字段是否有值
		if(null==entity.getOrderNum()){
			int count = baseMapper.selectCount(null);
			entity.setOrderNum(count);
		}
		//处理id为空
		entity.setId(null);
		//默认未删除
		entity.setDelFlag(false);
		return super.saveOrUpdate(entity);
	}

	@Transactional
	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {

		for (Serializable ids  :idList){
			OrganizationEvaluationRule rule = baseMapper.selectById(ids);
			if(null == rule){
				throw new CustomException("查无该记录");
			}
			rule.setDelFlag(true);
			rule.setUpdateTime(new Date());
			int num = baseMapper.updateById(rule);
			if (num<=0){
				throw new CustomException("删除失败");
			}
		}
		return true;
	}
}
