package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.ResponsibilityIndicatorInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.dao.OrganizationEvaluationMapper;
import com.ims.eval.entity.dto.request.IndicatorDTO;
import com.ims.eval.entity.dto.request.IndicatorDictionaryDTO;
import com.ims.eval.entity.dto.response.*;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 单位/部门考评记录 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-16
 */
@Service
public class OrganizationEvaluationServiceImpl extends ServiceImpl<OrganizationEvaluationMapper, OrganizationEvaluation> implements IOrganizationEvaluationService {


	/**
	 * 指标主业务类
	 */
	@Autowired
	private IIndicatorService iIndicatorService;

	/**
	 * 指标子业务类
	 */
	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;


	/**
	 * 单位/部门考评配置业务类
	 */
	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	/**
	 * 考评规则业务类
	 */
	@Autowired
	private IEvaluateRuleService evaluateRuleService;

	/**
	 * 考评规则明细业务类
	 */
	@Autowired
	private IEvaluateRuleInfoService evaluateRuleInfoService;


	@Resource
	private ResponsibilityIndicatorInfoMapper responsibilityIndicatorInfoMapper;

	@Resource
	private IOrganizationEvaluationInfoService organizationEvaluationInfoService;


	@Resource
	private IDeptResponsibilityService deptResponsibilityService;

	@Resource
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;

	@Resource
	private IEvaluationScoreCountService evaluationScoreCountService;

	@Resource
	private IBinSectionService binSectionService;

	@Autowired
	private IEvaluateReportService evaluateReportService;


	@Override
	public IPage<OrganizationEvaluation> list(Integer pageNum, Integer pageSize, String id, String organizationEvaluationCode, List<String> checkCycle, String beginDate, String endDate, String stage, String createBy, String year, String month, String des) {
		QueryWrapper<OrganizationEvaluation> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<OrganizationEvaluation> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(OrganizationEvaluation::getId, id);
		}

		if (StringUtils.isNotEmpty(organizationEvaluationCode)) {
			qw.lambda().like(OrganizationEvaluation::getOrganizationEvaluationCode, organizationEvaluationCode);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().in(OrganizationEvaluation::getCheckCycle, checkCycle);
		}

		if (StringUtils.isNotEmpty(beginDate) && StringUtils.isNotEmpty(endDate)) {
			qw.lambda().ge(OrganizationEvaluation::getBeginDate, beginDate);
			qw.lambda().le(OrganizationEvaluation::getEndDate, endDate);
		}

		if (StringUtils.isNotEmpty(stage)) {
			qw.lambda().eq(OrganizationEvaluation::getStage, stage);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(OrganizationEvaluation::getCreateBy, createBy);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(OrganizationEvaluation::getCreateBy, createBy);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(OrganizationEvaluation::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(OrganizationEvaluation::getMonth, month);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(OrganizationEvaluation::getDes, des);
		}

		qw.lambda().orderByDesc(OrganizationEvaluation::getCreateTime);

		IPage<OrganizationEvaluation> list = baseMapper.selectPage(page, qw);

		return list;
	}


	@Override
	public boolean saveOrUpdate(OrganizationEvaluation entity) {
		//获取指定部门的规则
		List<OrganizationEvaluationRule> list = organizationEvaluationRuleService.listByIsCheck(entity.getOrganizationType(), entity.getCheckCycle(), true, false);
		if (!"JDKP".equals(entity.getCheckCycle()) && (null == list || list.size() <= 0)) {
			throw new CustomException("不存在单位权重配置");
		}
		//获取组织id;
		List<String> organizationIds = list.stream().map(OrganizationEvaluationRule::getOrganizationId).collect(Collectors.toList());
		List<String> organizationNames = list.stream().map(OrganizationEvaluationRule::getOrganizationName).collect(Collectors.toList());
		List<String> evaluationRuleIds = list.stream().map(OrganizationEvaluationRule::getId).collect(Collectors.toList());

		StringBuilder responsibilityCode = new StringBuilder();
		responsibilityCode.append(entity.getOrganizationType()).append("_");
		responsibilityCode.append(entity.getCheckCycle()).append("_");

		//判断该是否存在目标责任书
		List<OrganizationEvaluation> oriList = new ArrayList<>();
		if ("NDKP".equals(entity.getCheckCycle())) {
			oriList = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(entity.getYear()).append("_");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(oriList.size() + 1), 4, "l"));
		}
		if ("JDKP".equals(entity.getCheckCycle())) {
			Date newDate = DateUtils.toDate(entity.getYear() + "-" + entity.getMonth() + "-01");
//			String season = String.valueOf(DateUtils.getSeason(newDate));//获取季度
//			String month = String.valueOf(Integer.valueOf(entity.getMonth()) * Integer.valueOf(season));//季度考评
			oriList = list("", entity.getCheckCycle(), entity.getYear(), entity.getMonth());
			List<OrganizationEvaluation> count = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(entity.getYear()).append("_").append(entity.getMonth()).append("_");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(count.size() + 1), 4, "l"));
		}
		if ("YDKP".equals(entity.getCheckCycle())) {
			oriList = list("", entity.getCheckCycle(), entity.getYear(), entity.getMonth());
			responsibilityCode.append(entity.getYear()).append("_").append(entity.getMonth()).append("_");
			List<OrganizationEvaluation> count = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(count.size() + 1), 4, "l"));
		}


		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			if (null != oriList && oriList.size() > 0) {
				throw new CustomException("此单位在该阶段已存在考评记录");
			}
			//保存目标责任书
			entity.setOrganizationEvaluationCode(responsibilityCode.toString().toLowerCase());

			entity.setOrganizationId(String.join(",", organizationIds));//考评组织ID
			entity.setOrganizationName(String.join(",", organizationNames));
			entity.setOrganizationEvaluationRuleId(String.join(",", evaluationRuleIds));
			entity.setStage("流程未启动");
			entity.setCreateTime(new Date());
		} else {
			OrganizationEvaluation update = baseMapper.selectById(entity.getId());
			if (null != oriList && oriList.size() > 1 && !update.getId().equals(oriList.get(0).getId())) {
				throw new CustomException("编辑记录未找到");
			}
			if (!"流程未启动".equals(update.getStage())) {
				throw new CustomException("流程已启动不允许修改");
			}
			entity.setOrganizationId(String.join(",", organizationIds));//考评组织ID
			entity.setOrganizationName(String.join(",", organizationNames));
			entity.setOrganizationEvaluationRuleId(String.join(",", evaluationRuleIds));
			entity.setUpdateTime(new Date());
		}
		boolean b = super.saveOrUpdate(entity);
		return b;
	}




	@Transactional
	@Override
	public boolean generateUnitEvaluationInfo(List<String> ids) {

		for (String id : ids) {
			OrganizationEvaluation organizationEvaluation = baseMapper.selectById(id);
			if (null == organizationEvaluation) {
				throw new CustomException("不存在考评记录");
			}

			//查询对应年度的目标责任书
			List<DeptResponsibility> responsibilities = deptResponsibilityService.list("", organizationEvaluation.getCheckCycle(), organizationEvaluation.getYear(), "");
			if (null == responsibilities || responsibilities.size() <= 0) {
				throw new CustomException("该年度的目标责任书不存在");
			}
			DeptResponsibility deptResponsibility = responsibilities.get(0);

			//检查是否已经启动考评
			QueryWrapper<OrganizationEvaluationInfo> infoqw = new QueryWrapper<>();
			infoqw.lambda().eq(OrganizationEvaluationInfo::getOrganizationEvaluationId, id);
			int infcount = organizationEvaluationInfoService.count(infoqw);
			if (infcount > 0) {
				throw new CustomException("编号" + organizationEvaluation.getOrganizationEvaluationCode() + "的考评已启动;不允许重复操作");
			}

			organizationEvaluation.setStage("流程启动");
			baseMapper.updateById(organizationEvaluation);

			List<String> evaluationRuleIds = Arrays.asList(organizationEvaluation.getOrganizationEvaluationRuleId().split(","));

			for (String ruleId : evaluationRuleIds) {

				//获取指定部门的规则
				OrganizationEvaluationRule evaluationRule = organizationEvaluationRuleService.getByIdIsCheck(ruleId, true, false);
				if (null == evaluationRule) {
					throw new CustomException("不存在此单位规则权重配置");
				}
				//获取对应的考评规（至少存在一条）
				List<String> evaluateRuleIds = Arrays.asList(evaluationRule.getEvaluateRuleId().split(","));
				List<EvaluateRule> evaluateRuleList = evaluateRuleService.listAll(evaluateRuleIds);
				if (null == evaluateRuleList || evaluateRuleList.size() <= 0) {
					throw new CustomException("未获取到配置的规则");
				}
				//获取考评规则的id
				List<String> ruleIds = evaluateRuleList.stream().map(EvaluateRule::getId).collect(Collectors.toList());
				//通过规则id获取对应的规则明细
				List<EvaluateRuleInfo> ruleInfos = evaluateRuleInfoService.list(ruleIds);
				if (null == ruleInfos || ruleInfos.size() <= 0) {
					throw new CustomException("规则明细配置为空");
				}
				//获取考评规则中的指标id
				List<String> indicatorIds = ruleInfos.stream().map(EvaluateRuleInfo::getIndicatorId).collect(Collectors.toList());

				//获取指标对象
//				List<Indicator> indicators = iIndicatorService.selectByIds(indicatorIds);
				//通过指标id获取指标明细项
				List<IndicatorResDTO> resDTOS = iIndicatorService.listByIds(indicatorIds);
				if (null == resDTOS || resDTOS.size() <= 0) {
					throw new CustomException("指标规则明细项为空");
				}

				//对应指标的目标值
				Map<String, Object> objectMap = responsibilityIndicatorInfoService.getResponsibilityIdMap(deptResponsibility.getId());

				resDTOS.stream().forEach(r -> {
					OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
					info.setOrganizationEvaluationId(organizationEvaluation.getId());
					info.setOrganizationEvaluationRuleId(ruleId);
					info.setIndicatorId(r.getId());
					info.setDeptId(r.getDeptId());
					info.setChildCode(r.getChildCode());
					info.setIndicatorDictionaryId(r.getIndicatorDictionaryID());
					info.setOptionCode(r.getOptionCode());
					info.setIsQuantified(r.getIsQuantified());
					//
					if ("JHZ".equals(r.getOptionCode()) || "MBZ".equals(r.getOptionCode()) || "DBZ".equals(r.getOptionCode())) {
						if (null != objectMap.get(r.getId())) {
							if ("是".equals(r.getIsQuantified())) {
								info.setQuantifiedValue(null == objectMap.get(r.getId()) ? null : Double.valueOf(objectMap.get(r.getId()).toString()));
							} else {
								info.setNonQuantifiedValue(null == objectMap.get(r.getId()) ? null : objectMap.get(r.getId()).toString());
							}
						}
					}
					info.setCreateTime(new Date());
					boolean b2 = organizationEvaluationInfoService.saveOrUpdate(info);
					if (!b2) {
						throw new CustomException("初始化目标责任书明细失败");
					}
				});
			}
		}

		return true;
	}

	@Transactional
	@Override
	public boolean generateUnitMonthEvaluation(List<String> ids) {

		for (String id : ids) {
			OrganizationEvaluation organizationEvaluation = baseMapper.selectById(id);
			if (null == organizationEvaluation) {
				throw new CustomException("不存在考评记录");
			}

			String organizationType = organizationEvaluation.getOrganizationType();//考评类型 DWKP
			String checkCycle = organizationEvaluation.getCheckCycle();//考评周期-月度、季度
			String year = organizationEvaluation.getYear();//年份

			//查询当前周期下对应的考评权重（需要考评的单位）
			List<OrganizationEvaluationRule> evaluationRules = organizationEvaluationRuleService.getOrganizationEvaluationRuleByYearAndCycle(organizationType,checkCycle,year);


			//遍历考核的单位
			for(OrganizationEvaluationRule evaluationRule : evaluationRules){
				//规则集合
				List<EvaluateRule> evaluateRuleList = new ArrayList<>();
				//根据权重获取对应的考评规则
				//生产经营
				double scjyWeight = evaluationRule.getScjyWeight();
				if(scjyWeight>0){
					List<EvaluateRule> scjyevaluateRuleList =  evaluateRuleService.getEvaluateRuleByYear(evaluationRule.getBinSection(),CacheContext.bseCodeObject.get("SCJY").getId(),"JDKP",evaluationRule.getYear());
					if(null!=scjyevaluateRuleList &&scjyevaluateRuleList.size()>0){
						evaluateRuleList.addAll(scjyevaluateRuleList);
					}
				}
				//前期
				double qqWeight = evaluationRule.getQqWeight();
				if(qqWeight>0){
					List<EvaluateRule> qqevaluateRuleList = evaluateRuleService.getEvaluateRuleByYear(CacheContext.bsnCodeObject.get("QQ").getId(),CacheContext.bseCodeObject.get("QQXM").getId(),"JDKP",evaluationRule.getYear());
					if(null!=qqevaluateRuleList &&qqevaluateRuleList.size()>0){
						evaluateRuleList.addAll(qqevaluateRuleList);
					}
				}
				//基建
				double jjWeight = evaluationRule.getJjWeight();
				if(jjWeight>0){
					List<EvaluateRule> jjevaluateRuleList = evaluateRuleService.getEvaluateRuleByYear(CacheContext.bsnCodeObject.get("JJ").getId(),CacheContext.bseCodeObject.get("JJXM").getId(),"JDKP",evaluationRule.getYear());
					if(null!=jjevaluateRuleList &&jjevaluateRuleList.size()>0){
						evaluateRuleList.addAll(jjevaluateRuleList);
					}
				}
				//获取考评规则的id
				List<String> ruleIds = evaluateRuleList.stream().map(EvaluateRule::getId).collect(Collectors.toList());
				//通过规则id获取对应的规则明细
				if(ruleIds.size()<=0){
					continue;
				}
				List<EvaluateRuleInfo> ruleInfos = evaluateRuleInfoService.list(ruleIds);
				if (null == ruleInfos || ruleInfos.size() <= 0) {
					continue;
				}
				//获取考评规则中的规则明细id
				List<String> ruleInfoIds = ruleInfos.stream().map(EvaluateRuleInfo::getId).collect(Collectors.toList());

				//通过规则id获取指标明细项
				List<EvaluateRuleInfoIndicatorDTO> resDTOS = evaluateRuleInfoService.listByIndicatorInfo(ruleInfoIds);

				if (null == resDTOS || resDTOS.size() <= 0) {
					throw new CustomException("指标规则明细项为空");
				}
				List<OrganizationEvaluationInfo> organizationEvaluationInfos = new ArrayList<>();
				for (EvaluateRuleInfoIndicatorDTO r : resDTOS) {
					if (null == r.getIndicatorDictionaryID()) {
						continue;
					}
					OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
					info.setOrganizationEvaluationId(organizationEvaluation.getId());
					info.setOrganizationEvaluationRuleId(evaluationRule.getId());
					info.setEvaluateRuleInfoId(r.getId());
					info.setIndicatorId(r.getIndicatorId());
					info.setChildCode(r.getChildCode());
					info.setDeptId(r.getDeptId());
					info.setIndicatorDictionaryId(r.getIndicatorDictionaryID());
					info.setOptionCode(r.getOptionCode());
					info.setIsQuantified(r.getIsQuantified());
					info.setCreateTime(new Date());
					organizationEvaluationInfos.add(info);
				}
				boolean b2 = organizationEvaluationInfoService.saveBatch(organizationEvaluationInfos);
				if (!b2) {
					throw new CustomException("初始化目标责任书明细失败");
				}
			}
		}
		return true;
	}

	@Transactional
	@Override
	public boolean generateUnitSeasonEvaluation(List<String> ids) {


		for (String id : ids) {
			OrganizationEvaluation organizationEvaluation = baseMapper.selectById(id);
			if (null == organizationEvaluation) {
				throw new CustomException("不存在考评记录");
			}

			//检查是否已经启动考评
			QueryWrapper<OrganizationEvaluationInfo> infoqw = new QueryWrapper<>();
			infoqw.lambda().eq(OrganizationEvaluationInfo::getOrganizationEvaluationId, id);
			int infcount = organizationEvaluationInfoService.count(infoqw);
			if (infcount > 0) {
				throw new CustomException("编号" + organizationEvaluation.getOrganizationEvaluationCode() + "的考评已启动;不允许重复操作");
			}

			organizationEvaluation.setStage("流程启动");
			baseMapper.updateById(organizationEvaluation);

			List<String> evaluationRuleIds = Arrays.asList(organizationEvaluation.getOrganizationEvaluationRuleId().split(","));
			evaluationRuleIds = evaluationRuleIds.stream().distinct().collect(Collectors.toList());
			for (String ruleId : evaluationRuleIds) {

				//获取指定部门的规则
				OrganizationEvaluationRule evaluationRule = organizationEvaluationRuleService.getById(ruleId);
				if (null == evaluationRule || evaluationRule.getDelFlag().equals(true)) {
					throw new CustomException(evaluationRule.getOrganizationShortName() + "不存在此单位规则权重配置");
				}
				//获取对应的考评规（至少存在一条）
				List<String> evaluateRuleIds = Arrays.asList(evaluationRule.getEvaluateRuleId().split(","));
				List<EvaluateRule> evaluateRuleList = evaluateRuleService.listAll(evaluateRuleIds);
				if (null == evaluateRuleList || evaluateRuleList.size() <= 0) {
					//throw new CustomException("未获取到配置的规则");
					continue;
				}
				//获取考评规则的id
				List<String> ruleIds = evaluateRuleList.stream().map(EvaluateRule::getId).collect(Collectors.toList());
				//通过规则id获取对应的规则明细
				List<EvaluateRuleInfo> ruleInfos = evaluateRuleInfoService.list(ruleIds);
				if (null == ruleInfos || ruleInfos.size() <= 0) {
//					throw new CustomException("规则明细配置为空");
					continue;
				}
				//获取考评规则中的规则明细id
				List<String> ruleInfoIds = ruleInfos.stream().map(EvaluateRuleInfo::getId).collect(Collectors.toList());

				//通过规则id获取指标明细项
				List<EvaluateRuleInfoIndicatorDTO> resDTOS = evaluateRuleInfoService.listByIndicatorInfo(ruleInfoIds);


				if (null == resDTOS || resDTOS.size() <= 0) {
					throw new CustomException("指标规则明细项为空");
				}
				for (EvaluateRuleInfoIndicatorDTO r : resDTOS) {
					if (null == r.getIndicatorDictionaryID()) {
						continue;
					}
					OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
					info.setOrganizationEvaluationId(organizationEvaluation.getId());
					info.setOrganizationEvaluationRuleId(ruleId);
					info.setEvaluateRuleInfoId(r.getId());
					info.setIndicatorId(r.getIndicatorId());
					info.setChildCode(r.getChildCode());
					info.setDeptId(r.getDeptId());
					info.setIndicatorDictionaryId(r.getIndicatorDictionaryID());
					info.setOptionCode(r.getOptionCode());
					info.setIsQuantified(r.getIsQuantified());
					info.setCreateTime(new Date());
					boolean b2 = organizationEvaluationInfoService.saveOrUpdate(info);
					if (!b2) {
						throw new CustomException("初始化目标责任书明细失败");
					}
				}
			}
		}

		return true;
	}


	@Override
	public List<OrganizationEvaluation> list(String organizationId, String checkCycle, String year, String month) {
		QueryWrapper<OrganizationEvaluation> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(OrganizationEvaluation::getOrganizationId, organizationId);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(OrganizationEvaluation::getCheckCycle, checkCycle);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(OrganizationEvaluation::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().in(OrganizationEvaluation::getMonth, month);
		}
		List<OrganizationEvaluation> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		OrganizationEvaluation evaluation = baseMapper.selectById(id);

		if("人事部审核".equals(state)){
			state = "流程结束";

			//流程结束  计算相应的得分和汇总报告
			boolean b1 = organizationEvaluationInfoService.calculationByEvaluationId("","",id,"");
			if(b1){
				boolean b2 = evaluateReportService.generateEvaluateReport(id,false);
			}

		}
		if (StringUtils.isNotEmpty(state)) {
			evaluation.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			evaluation.setInstId(instId);
		}
		boolean b = super.saveOrUpdate(evaluation);
		return b;
	}


	@Override
	public OrganizationEvaluationResDTO getByidAndInfo(String id, String dept, HttpServletRequest request) {
		OrganizationEvaluationResDTO resDTO = baseMapper.selectById(id);
		Map<String, List<OrganizationEvaluationInfoResDTO>> map = organizationEvaluationInfoService.finishValueList(id, dept, request);
		if (null != resDTO) {
			resDTO.setMap(map);
		}
		return resDTO;
	}

	@Override
	public List<Map<String, Object>> getCompanyWarning(String evaluationType) {
		return baseMapper.selectCompanyActualValue(null, null, evaluationType);
	}


	@Transactional
	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {

		QueryWrapper<OrganizationEvaluationInfo> infoqw = new QueryWrapper<>();
		infoqw.lambda().in(OrganizationEvaluationInfo::getOrganizationEvaluationId, idList);
		int infocount = organizationEvaluationInfoService.count(infoqw);
		if (infocount > 0) {
			throw new CustomException("已发起流程不允许删除");
		}
		boolean b = super.removeByIds(idList);
		if (!b) {
			throw new CustomException("删除失败");
		}
		return true;
	}

	@Override
	public Map getEvaluationScoreInfo(String id, String binSection, String indicatorId, String organizationEvaluationRuleId) {

		BinSection section =  CacheContext.bsnIdObject.get(binSection);

		Map map = new HashMap();
		String title1 = "";
		List<Object> indicatorNamelist = new ArrayList<>();
		Map<String, Object> indicatorName = new LinkedHashMap<>();
		List<Object> bodylist = new ArrayList<>();

		//获取到数据
		List<EvaluationScoreInfoResDTO> listdt = baseMapper.selectEvaluationScoreInfoList(id, binSection, indicatorId, organizationEvaluationRuleId);

		if(null == listdt || listdt.size()<=0){
			throw new CustomException("未找到对应的打分记录");
		}

		//1.指标id是否为空（有值显示当前指标打分）
		if (StringUtils.isNotEmpty(indicatorId)) {


			IndicatorDTO indicator = iIndicatorService.getById(indicatorId);
			title1 = indicator.getIndicatorName()+"打分表";

			List<Object> childlist = new ArrayList<>();
			List<IndicatorDictionaryDTO> indicatorDictionaryDTOList = indicator.getDtoList();
			for(IndicatorDictionaryDTO dto:indicatorDictionaryDTOList){
				Map<String, Object> childName = new LinkedHashMap<>();
				List<Object> oplist = new ArrayList<>();
				for(IndicatorDictionary ids : dto.getList()){
					if(!ids.getIsShow()){
						continue;
					}
					Map<String, Object> opmap = new HashMap<>();
					opmap.put("key", dto.getChildCode() + "_" + ids.getOptionCode());
					opmap.put("name", ids.getOptionName());
					oplist.add(opmap);
				}
				//有子指标和没有子指标区别
				if(indicatorDictionaryDTOList.size()>1){
					childName.put(dto.getChildName(),oplist);
					childlist.add(childName);
				}else {
					childlist.addAll(oplist);
				}

			}

			indicatorName.put(indicator.getIndicatorName(),childlist);

			//按照公司分组
			Map<String, List<EvaluationScoreInfoResDTO>> organizationScoreMap = listdt.stream().collect(Collectors.groupingBy(score->score.getOrganizationId()+","+score.getOrganizationShortName()));
			for (Map.Entry<String, List<EvaluationScoreInfoResDTO>> organizationScoreEntry : organizationScoreMap.entrySet()) {
				Map<String, Object> opmap = new HashMap<>();
				//公司名
				opmap.put("organizationShortName",organizationScoreEntry.getKey().split(",")[1]);
				//按照子指标分组
				Map<String, List<EvaluationScoreInfoResDTO>> childNameMap = organizationScoreEntry.getValue().stream().collect(Collectors.groupingBy(EvaluationScoreInfoResDTO::getChildCode));
				for (Map.Entry<String, List<EvaluationScoreInfoResDTO>> childNameEntry : childNameMap.entrySet()) {
					List<EvaluationScoreInfoResDTO> listdto = childNameEntry.getValue();
					Boolean zhdfflg = true;
					for (EvaluationScoreInfoResDTO dto : listdto) {
						//综合得分
						if(zhdfflg && (childNameMap.size()>1 || dto.getChildName().equals("净利润"))){
							List<EvaluationScoreCount> scoreCounts = evaluationScoreCountService.getEvaluationScoreCountList(id,dto.getOrganizationEvaluationRuleId(),dto.getIndicatorId());
							opmap.put("zhdf",scoreCounts.get(0).getObversionScore());
							zhdfflg = false;
						}
						opmap.put(dto.getChildCode()+"_"+dto.getOptionCode(),dto.getQuantifiedValue());
					}
				}
				bodylist.add(opmap);
			}

			Map<String, Object> title = new LinkedHashMap<>();
			title.put(title1,indicatorName);
			map.put("title",title);
			map.put("values",bodylist);

		} else {
			//2.指标id为空（显示所有指标打分）

			if(section.getSectionCode().equals("JJ")){
				title1 = section.getSectionName()+"项目指标打分表（工程建设部）";

			}else if(section.getSectionCode().equals("QQ")){
				title1 = section.getSectionName()+"项目指标打分表（计划发展部）";
			}else {
				title1 = section.getSectionName()+"企业生产指标考核打分表";
			}

			List<String> ruleIds = listdt.stream().map(EvaluationScoreInfoResDTO::getEvaluateRuleId).collect(Collectors.toList());
			ruleIds = ruleIds.stream().distinct().collect(Collectors.toList());
			if(ruleIds.size()>1){
				return null;
			}
			List<EvaluateRuleInfo> ruleInfos = evaluateRuleInfoService.list(ruleIds);
			List<String> indicatorIds = ruleInfos.stream().map(EvaluateRuleInfo::getIndicatorId).distinct().collect(Collectors.toList());
			for (String i : indicatorIds) {
				Map<String, Object> indicatorName2 = new LinkedHashMap<>();
				IndicatorDTO indicator = iIndicatorService.getById(i);

				List<Object> childlist = new ArrayList<>();
				List<IndicatorDictionaryDTO> indicatorDictionaryDTOList = indicator.getDtoList();
				Boolean zhdfflg = true;
				int count = 0;
				for (IndicatorDictionaryDTO dto : indicatorDictionaryDTOList) {
					count++;
					if(count>1){
						continue;
					}
					List<Object> oplist = new ArrayList<>();
					for (IndicatorDictionary ids : dto.getList()) {
						if (!ids.getIsShow() || ids.getOptionCode().equals("CZ")) {
							continue;
						}
						Map<String, Object> opmap = new HashMap<>();
						opmap.put("key", dto.getChildCode() + "_" + ids.getOptionCode());
						opmap.put("name", ids.getOptionName());

						oplist.add(opmap);
					}
					//有子指标和没有子指标区别
					if (zhdfflg && (dto.getChildName().equals("净利润"))) {
						Map<String, Object> zhdf = new HashMap<>();
						zhdf.put("key", dto.getChildCode()+"_zhdf");
						zhdf.put("name", "综合得分");
						zhdfflg = false;
						oplist.add(zhdf);
					}

					childlist.addAll(oplist);

				}

				indicatorName2.put(indicator.getIndicatorName(), childlist);
				indicatorNamelist.add(indicatorName2);
			}


			List<Object> bodylist2 = new ArrayList<>();
			//按照公司分组
			Map<String, List<EvaluationScoreInfoResDTO>> organizationScoreMap = listdt.stream().collect(Collectors.groupingBy(score->score.getOrganizationId()+","+score.getOrganizationShortName()));


			for (Map.Entry<String, List<EvaluationScoreInfoResDTO>> organizationScoreEntry : organizationScoreMap.entrySet()) {
				Map<String, Object> opmap = new HashMap<>();
				//公司名
				opmap.put("organizationShortName",organizationScoreEntry.getKey().split(",")[1]);
				//按照子指标分组
				Map<String, List<EvaluationScoreInfoResDTO>> indicatorIdMap = organizationScoreEntry.getValue().stream().collect(Collectors.groupingBy(EvaluationScoreInfoResDTO::getIndicatorId));


				Boolean zhdfflg2 = true;
				for (Map.Entry<String, List<EvaluationScoreInfoResDTO>> indicatorIdEntry : indicatorIdMap.entrySet()) {
					Map<String, List<EvaluationScoreInfoResDTO>> iChildCodeMap = indicatorIdEntry.getValue().stream().collect(Collectors.groupingBy(EvaluationScoreInfoResDTO::getChildCode));

					for (EvaluationScoreInfoResDTO dto : indicatorIdEntry.getValue()) {
						if(iChildCodeMap.size()>1 && dto.getOptionCode().equals("DF")){
							List<EvaluationScoreCount> scoreCounts = evaluationScoreCountService.getEvaluationScoreCountList(id,dto.getOrganizationEvaluationRuleId(),dto.getIndicatorId());
							opmap.put(dto.getChildCode()+"_"+dto.getOptionCode(),scoreCounts.get(0).getObversionScore());
						}else {
							opmap.put(dto.getChildCode()+"_"+dto.getOptionCode(),dto.getQuantifiedValue());
						}

						//综合得分
						if(zhdfflg2 && (dto.getChildName().equals("净利润"))){
							List<EvaluationScoreCount> scoreCounts = evaluationScoreCountService.getEvaluationScoreCountList(id,dto.getOrganizationEvaluationRuleId(),dto.getIndicatorId());
							opmap.put(dto.getChildCode()+"_zhdf",scoreCounts.get(0).getObversionScore());
							zhdfflg2 = false;
						}
					}
				}
				List<EvaluationScoreCount> scoreCounts2 = evaluationScoreCountService.getEvaluationScoreCountList(id,organizationScoreEntry.getValue().get(0).getOrganizationEvaluationRuleId(),"");
				Double scoresum =  scoreCounts2.stream().mapToDouble(EvaluationScoreCount::getObversionScore).sum();


				opmap.put("zhdf",new BigDecimal(scoresum).setScale(2, RoundingMode.HALF_UP).doubleValue());
				bodylist2.add(opmap);
			}

			Map<String, Object> title1map = new LinkedHashMap<>();
			Map<String, Object> title2map = new LinkedHashMap<>();
			title2map.put("考核指标名称",indicatorNamelist);
			title1map.put(title1,title2map);
			map.put("zhdf",true);
			map.put("title",title1map);
			map.put("values",bodylist2);
		}
		return map;
	}
}
