package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.PersonnelEvaluationRule;
import com.ims.eval.dao.PersonnelEvaluationRuleMapper;
import com.ims.eval.service.IPersonnelEvaluationRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-01
 */
@Service
public class PersonnelEvaluationRuleServiceImpl extends ServiceImpl<PersonnelEvaluationRuleMapper, PersonnelEvaluationRule> implements IPersonnelEvaluationRuleService {

	@Override
	public IPage<PersonnelEvaluationRule> list(Integer pageNum, Integer pageSize, String id, String personnelId, String personnelName, String companyId, String companyName, String deptId, String position, String evaluationCycle, String year, String month) {
		QueryWrapper<PersonnelEvaluationRule> qw = new QueryWrapper<>();

		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<PersonnelEvaluationRule> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(PersonnelEvaluationRule::getId, id);
		}

		if (StringUtils.isNotEmpty(personnelId)) {
			qw.lambda().like(PersonnelEvaluationRule::getPersonnelId, personnelId);
		}

		if (StringUtils.isNotEmpty(personnelName)) {
			qw.lambda().like(PersonnelEvaluationRule::getPersonnelName, personnelName);
		}

		if (StringUtils.isNotEmpty(companyId)) {
			qw.lambda().eq(PersonnelEvaluationRule::getCompanyId, companyId);
		}

		if (StringUtils.isNotEmpty(companyName)) {
			qw.lambda().like(PersonnelEvaluationRule::getCompanyName, companyName);
		}

		if (StringUtils.isNotEmpty(deptId)) {
			qw.lambda().eq(PersonnelEvaluationRule::getDeptId, deptId);
		}

		if (StringUtils.isNotEmpty(position)) {
			qw.lambda().eq(PersonnelEvaluationRule::getPosition, position);
		}
		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(PersonnelEvaluationRule::getEvaluationCycle, evaluationCycle);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(PersonnelEvaluationRule::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(PersonnelEvaluationRule::getMonth, month);
		}
		qw.lambda().orderByAsc(PersonnelEvaluationRule::getOrderNum);

		IPage<PersonnelEvaluationRule> list = baseMapper.selectPage(page, qw);

		return list;
	}

	@Override
	public List<PersonnelEvaluationRule> listAll(String id, String personnelId, String personnelName, String companyId, String companyName, String deptId, String position, String evaluationCycle, String year, String month) {
		QueryWrapper<PersonnelEvaluationRule> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(PersonnelEvaluationRule::getId, id);
		}

		if (StringUtils.isNotEmpty(personnelId)) {
			qw.lambda().like(PersonnelEvaluationRule::getPersonnelId, personnelId);
		}

		if (StringUtils.isNotEmpty(personnelName)) {
			qw.lambda().like(PersonnelEvaluationRule::getPersonnelName, personnelName);
		}

		if (StringUtils.isNotEmpty(companyId)) {
			qw.lambda().eq(PersonnelEvaluationRule::getCompanyId, companyId);
		}

		if (StringUtils.isNotEmpty(companyName)) {
			qw.lambda().like(PersonnelEvaluationRule::getCompanyName, companyName);
		}

		if (StringUtils.isNotEmpty(deptId)) {
			qw.lambda().eq(PersonnelEvaluationRule::getDeptId, deptId);
		}

		if (StringUtils.isNotEmpty(position)) {
			qw.lambda().eq(PersonnelEvaluationRule::getPosition, position);
		}
		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(PersonnelEvaluationRule::getEvaluationCycle, evaluationCycle);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(PersonnelEvaluationRule::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(PersonnelEvaluationRule::getMonth, month);
		}
		qw.lambda().orderByAsc(PersonnelEvaluationRule::getOrderNum);

		List<PersonnelEvaluationRule> list = baseMapper.selectList(qw);

		return list;
	}

	@Override
	public boolean saveOrUpdate(PersonnelEvaluationRule entity) {

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<PersonnelEvaluationRule> qw = new QueryWrapper<>();
			qw.lambda().eq(PersonnelEvaluationRule::getYear, entity.getYear());
			List<PersonnelEvaluationRule> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前指标code已存在");
			}
			entity.setCreateTime(new Date());
		} else {
			entity.setUpdateTime(new Date());
		}

		return super.saveOrUpdate(entity);
	}
}
