package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.Constant;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.ResponsibilityIndicatorInfoMapper;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.ResponsibilityIndicatorInfo;
import com.ims.eval.entity.dto.request.ResponsibilityIndicatorInfoUpdateDTO;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.service.IIndicatorService;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import com.ims.eval.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 责任书指标明细
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-08
 */
@Service
public class ResponsibilityIndicatorInfoServiceImpl extends ServiceImpl<ResponsibilityIndicatorInfoMapper, ResponsibilityIndicatorInfo> implements IResponsibilityIndicatorInfoService {


	@Autowired
	private IUserService userService;

	@Autowired
	private IIndicatorService indicatorService;

	@Override
	public Map<String, List<ResponsibilityIndicatorInfoResDTO>> planValueList(List<String> deptResponsibilityIds, String dept, HttpServletRequest request) {

		List<ResponsibilityIndicatorInfoResDTO> list = baseMapper.listByresponsibilityId(deptResponsibilityIds, dept, "ZRMB", "");//默认获取指定的怎目标

		Map<String, List<ResponsibilityIndicatorInfoResDTO>> map = list.stream().collect(Collectors.groupingBy(ResponsibilityIndicatorInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove("是"));
		map.put("nonQuantifiedList", map.remove("否"));
		return map;

	}

	@Override
	public Map<String, List<ResponsibilityIndicatorInfoResDTO>> finishValueList(List<String> deptResponsibilityIds, String dept) {


		return null;
	}

	@Override
	public Map<String, List<ResponsibilityIndicatorInfoResDTO>> selectResponsibilityIndicatorInfoList(String deptResponsibilityId, String isQuantified, String dataState) {
		List<ResponsibilityIndicatorInfoResDTO> list = baseMapper.selectResponsibilityIndicatorInfoList(deptResponsibilityId, isQuantified, dataState);
		Map<String, List<ResponsibilityIndicatorInfoResDTO>> map = list.stream().collect(Collectors.groupingBy(ResponsibilityIndicatorInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove("是"));
		map.put("nonQuantifiedList", map.remove("否"));
		return map;
	}


	@Transactional
	@Override
	public boolean saveOrUpdateDto(ResponsibilityIndicatorInfoUpdateDTO infoUpdateDTO) {
		ResponsibilityIndicatorInfo indicatorInfo = baseMapper.selectById(infoUpdateDTO.getId());
		indicatorInfo.setQuantifiedValue(infoUpdateDTO.getQuantifiedValue());//量化值
		indicatorInfo.setNonQuantifiedValue(infoUpdateDTO.getNonQuantifiedValue());//非量化值
		indicatorInfo.setUpdateTime(new Date());//更新时间
		indicatorInfo.setRemark(infoUpdateDTO.getRemark());//dicatorInfo.setUpdateBy(infoUpdateDTO.getUpdateBy());//更新者备注
		return super.saveOrUpdate(indicatorInfo);
	}


	@Transactional
	@Override
	public boolean saveBatchDto(List<ResponsibilityIndicatorInfoUpdateDTO> infoUpdateDTOs) {
		for (ResponsibilityIndicatorInfoUpdateDTO entity : infoUpdateDTOs) {

			if (null != entity && null != entity.getId() && !"".equals(entity.getId().trim())) {
				ResponsibilityIndicatorInfo indicatorInfo = baseMapper.selectById(entity.getId());

				indicatorInfo.setQuantifiedValue(entity.getQuantifiedValue());//量化值
				indicatorInfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());//非量化值
				indicatorInfo.setUpdateTime(new Date());//更新时间
				indicatorInfo.setUpdateBy(entity.getUpdateBy());//更新者
				indicatorInfo.setRemark(entity.getRemark());//备注
				indicatorInfo.setState(entity.getState());//状态
				boolean b = super.saveOrUpdate(indicatorInfo);
				if (!b) {
					throw new CustomException("更新失败");
				}
			} else {
				ResponsibilityIndicatorInfo addInfo = new ResponsibilityIndicatorInfo();
				addInfo.setDeptResponsibilityId(entity.getDeptResponsibilityId());
				addInfo.setOrganizationEvaluationRuleId(entity.getOrganizationEvaluationRuleId());
				addInfo.setIndicatorId(entity.getIndicatorId());
//				addInfo.setIndicatorDictionaryId(entity.getIndicatorDictionaryId());
				addInfo.setOptionCode(entity.getOptionCode());
				addInfo.setIsQuantified(entity.getIsQuantified());
				addInfo.setQuantifiedValue(entity.getQuantifiedValue());
				addInfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());
				addInfo.setRemark(entity.getRemark());//备注
				addInfo.setCreateTime(new Date());
				addInfo.setState("-1");//状态

				Indicator indicator = indicatorService.getById(entity.getIndicatorId());
				addInfo.setDeptId(indicator.getDeptId());

				boolean b = super.saveOrUpdate(addInfo);
				if (!b) {
					throw new CustomException("新增失败");
				}
			}

		}
		return true;
	}

	@Override
	public Map<String, Object> getResponsibilityIdMap(String deptResponsibilityId) {
		Map<String, Object> map = new HashMap<>();
		QueryWrapper<ResponsibilityIndicatorInfo> qw = new QueryWrapper<>();
		qw.lambda().eq(ResponsibilityIndicatorInfo::getDeptResponsibilityId, deptResponsibilityId);
		List<ResponsibilityIndicatorInfo> list = baseMapper.selectList(qw);
		list.stream().forEach(i -> {
			map.put(i.getIndicatorId(), i.getIsQuantified().equals("是") ? i.getQuantifiedValue() : i.getQuantifiedValue());
		});
		return map;
	}

	@Override
	public boolean saveOrUpdateBatch(List<ResponsibilityIndicatorInfo> responsibilityIndicatorInfoList) {
		boolean b = false;
		for (ResponsibilityIndicatorInfo obj : responsibilityIndicatorInfoList) {
			ResponsibilityIndicatorInfo responsibilityIndicatorInfo = super.getById(obj.getId());
			if (!"失效".equals(responsibilityIndicatorInfo.getDataState())) {
				obj.setDataState("失效");
				b = super.updateById(obj);
				if (b) {
					ResponsibilityIndicatorInfo info = new ResponsibilityIndicatorInfo();
					info.setDeptResponsibilityId(responsibilityIndicatorInfo.getDeptResponsibilityId());
					info.setOrganizationEvaluationRuleId(responsibilityIndicatorInfo.getOrganizationEvaluationRuleId());
					info.setIndicatorId(responsibilityIndicatorInfo.getIndicatorId());
					info.setIndicatorDictionaryId(responsibilityIndicatorInfo.getIndicatorDictionaryId());
					info.setOptionCode(responsibilityIndicatorInfo.getOptionCode());
					info.setIsQuantified(obj.getIsQuantified());
					info.setQuantifiedValue(obj.getQuantifiedValue());
					info.setNonQuantifiedValue(obj.getNonQuantifiedValue());
					info.setCreateTime(new Date());
					info.setState(obj.getState());
					info.setDataState("有效");
					b = super.save(info);
				}
			} else {
				obj.setUpdateTime(new Date());
				b = super.updateById(obj);
			}
		}
		return b;
	}

	@Override
	public List<ResponsibilityIndicatorInfoResDTO> selectPopUpData(String deptResponsibilityId, String isQuantified, String dataState) {
		List<ResponsibilityIndicatorInfoResDTO> list = baseMapper.selectResponsibilityIndicatorInfoList(deptResponsibilityId, isQuantified, dataState);
		return list;
	}
}
