package com.ims.eval.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.config.ImaConfig;
import com.ims.eval.entity.Myuser;
import com.ims.eval.dao.UserMapper;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.feign.RemoteServiceBuilder;
import com.ims.eval.service.IUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import springfox.documentation.annotations.Cacheable;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-03
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, Myuser> implements IUserService {


	@Autowired
	private RemoteServiceBuilder remoteServiceBuilder;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private ImaConfig imaConfig;

	@Override
	public List<Myuser> listAll(String id, String orgId, String unitId) {

		QueryWrapper<Myuser> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(Myuser::getId, id);
		}

//		if (StringUtils.isNotEmpty(orgId)) {
//			qw.lambda().like(Myuser::getOrgId, orgId);
//		}
//
//		if (StringUtils.isNotEmpty(unitId)) {
//			qw.lambda().like(Myuser::getUnitId, unitId);
//		}

		List<Myuser> list = baseMapper.selectList(qw);

		return list;
	}

	@Override
	public Myuser getUser(String username) {
		QueryWrapper<Myuser> qw = new QueryWrapper<>();
		qw.lambda().eq(Myuser::getLoginName, username);
		List<Myuser> list = baseMapper.selectList(qw);
		if (list.size() <= 0) {
			return null;
		}
		return list.get(0);
	}

	@Override
	public void updateUser(Myuser user) {
		super.saveOrUpdate(user);
	}

	@Cacheable(value = "user_code")
	@Override
	public MyuserResDTO getSysUser(String code) {
		try{
			Object json = remoteServiceBuilder.getGatewayUrl().getSysUser(code);
			MyuserResDTO user = JSON.parseObject(json.toString(), MyuserResDTO.class);
			return user;
		} catch (Exception e){
			return null;
		}
	}

	@Override
	public MyuserResDTO getSysUser(HttpServletRequest request) {

		if(null == request.getHeader("code")){
			throw new CustomException("请求鉴权信息为空");
		}
		String code = request.getHeader("code");
		JSONObject json = remoteServiceBuilder.getGatewayUrl().getSysUser(code);
		MyuserResDTO user =JSONObject.parseObject(json.getJSONObject("data").toJSONString(), MyuserResDTO.class);
		return user;
	}

	@Override
	public JSONObject getCodeByToken(String token) {
		JSONObject json = remoteServiceBuilder.getGatewayUrl().getCodeByToken(token);
		return json;
	}



//	public static void main(String[] args) {
//		String str = "{id=6446e07166ff7e09e7833f966926f207, companyId=, orgId=23031010, loginName=testcwadmin, password=, no=, name=测试财务部领导, email=, phone=, mobile=, userType=, photo=, loginIp=, loginDate=, loginFlag=1, createBy=dfd3cfd2044fb94f9974f12fc4928828, createDate=2023-03-20 18:18:11, updateBy=dfd3cfd2044fb94f9974f12fc4928828, updateDate=2023-03-20 18:18:11, remarks=, delFlag=0, sort=92, posCode=, token=7aa7586bf918483d8be12cd899d49c64, partyCost=-1, gender=1, birthday=, education=, residence=, nation=, idCard=, userStatus=0, entranceFactory=, leaveFactory=, secondaryPassword=, directSuperior=, directSuperiorName=, faceImageUrl=, unitId=23031001, deptId=23031010, teamId=, outUnitId=, accountType=, mainAccountId=, nativePlace=, hangingArea=厂前区, isSpecial=0, specialDate=, specialWorkRole=, inDate=2023-03-20, race=0, cardNumber=, isExtUser=, address=, type=, jobLevel=, employeeGroup=, employeeSubgroup=, registeredResidenceType=, joinWorkHours=, timeOfEnteringShenhuaSystem=, timeOfEnteringSubsidiaryCompany=, politicsStatus=, joinPartyTime=, initialEducation=, initialDegree=, initialEducationSchoolOfGraduation=, initialEducationProfessional=, initialEducationTimeOfGraduation=, highestEducationSchoolOfGraduation=, highestEducationProfessional=, highestEducationTimeOfGraduation=, highestProfessionalTechnicalQualifications=, professionalTechnicalQualificationsLevel=, highestProfessionalTechnicalGetTime=, professionalWork=, skillLevel=, professionalQualificationGetTime=, expertInformationLevel=, professionalArea=, religiousBeliefs=, passportNo=, mdmCompanyCode=, mdmSecondCompanyCode=, dataSource=, postLevel=, salaryLevel=, rankSystem=, isAdminRole=true, unitName=国电电力发展股份有限公司本部, unitOrg={id=23031001, parentId=23031000, parentIds=,0,20000001,23031000,, name=国电电力发展股份有限公司本部, sort=0, areaId=, code=, orgCode=, status=, businessType=火电, type=160, grade=1, address=安慧北里安园19号, zipCode=100101, master=, phone=58685000, fax=64829900, email=, useable=true, primaryPerson=, lng=, lat=, siteId=, deputyPerson=, createBy=dfd3cfd2044fb94f9974f12fc4928828, createDate=2022-07-01 19:37:26, updateBy=dfd3cfd2044fb94f9974f12fc4928828, updateDate=2022-11-11 16:19:37, remarks=, delFlag=0, shortName=国电电力本部, classId=1, classId2=, star=, mapName=, isBm=0, isMarket=0, nativeNo=, isExtOrg=, mdmId=10051709, registerName=, parentPropertyManagementCode=, parentCompanyManagementCode=, registerCountry=, registerProvince=010 - 北京市, registerAddress=, registeredCapital=, companyLevel=, propertyLevel=, isVaild=否, isVaildHr=否, mdmIdHr=, mainBusinessType=燃煤发电, regUnitFlag=否, mdmCompanyCode=, mdmSecondUnitCode=23031000, sortStr=AA, hasChildren=false, parentName=, baseQuerySql=}, deptName=财务部, deptOrg={id=23031010, parentId=23031001, parentIds=,0,20000001,23031000,23031001,, name=财务部, sort=-1, areaId=, code=, orgCode=, status=, businessType=, type=140, grade=, address=, zipCode=, master=, phone=, fax=, email=, useable=true, primaryPerson=, lng=, lat=, siteId=, deputyPerson=, createBy=dfd3cfd2044fb94f9974f12fc4928828, createDate=2022-07-01 19:37:35, updateBy=dfd3cfd2044fb94f9974f12fc4928828, updateDate=2022-07-01 19:37:35, remarks=, delFlag=0, shortName=财务部, classId=, classId2=, star=, mapName=, isBm=0, isMarket=0, nativeNo=, isExtOrg=, mdmId=10051724, registerName=, parentPropertyManagementCode=, parentCompanyManagementCode=, registerCountry=, registerProvince=, registerAddress=, registeredCapital=, companyLevel=, propertyLevel=, isVaild=否, isVaildHr=否, mdmIdHr=, mainBusinessType=, regUnitFlag=否, mdmCompanyCode=, mdmSecondUnitCode=23031000, sortStr=AJ, hasChildren=false, parentName=, baseQuerySql=}, teamName=, teamOrg={}, outUnitName=, outUnitOrg={}, mainPos={}, faceImgEncode=, areaIds=, entranceFactoryDate=, leaveFactoryDate=, roleVOList=[], posVOList=[], commonToken=}";
//		Myuser user = JSON.parseObject(str,Myuser.class);
//		System.out.println(user.getDeptName());
//	}

	@Override
	public JSONObject getbladeAuth(String tenantId, String username, String password, String grantType, String scope, String type) {
		MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
		headers.add("Authorization", "Basic c2FiZXI6NWMwMmMzZDViNzYxNTNkZGM5ZTczYzc5YzMzNzYzODAxNmE5ZDM2Ng==");
		headers.add("Tenant-Id", "000000");
		HttpEntity<LinkedMultiValueMap<String, Object>> param = new HttpEntity<>(null, headers);

		ResponseEntity<String> responseEntity2 = restTemplate.postForEntity(imaConfig.getGatewayUrl() + "blade-auth/oauth/token?tenantId={1}&username={2}&password={3}&grant_type={4}&scope={5}&type={6}",
			param, String.class, tenantId, username, password, grantType, scope, type);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if(200== responseEntity2.getStatusCodeValue()){
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}

	@Override
	public JSONObject pageList(Integer current, Integer size, String orgId, String idCard, String mobile, String no, String name, String loginName,HttpServletRequest request) {

		MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
		headers.add("Blade-Auth", "bearer "+request.getHeader("Blade-Auth"));
		HttpEntity<LinkedMultiValueMap<String, Object>> param = new HttpEntity<>(null, headers);

		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getGatewayUrl() + "ims-idms/sysuser/page?current={1}&size={2}&orgId={3}&idCard={4}&mobile={5}&no={6}&name={7}&loginName={8}",
			HttpMethod.GET,param, String.class, current, size, orgId, idCard, mobile, no,name,loginName);

		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		//
		if (200 == responseEntity2.getStatusCodeValue()) {

			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;

	}
}
