package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.YearOperatingCoefficientMapper;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.YearGroupCoefficient;
import com.ims.eval.dao.YearGroupCoefficientMapper;
import com.ims.eval.entity.YearOperatingCoefficient;
import com.ims.eval.entity.dto.request.YearGroupCoefficientDTO;
import com.ims.eval.service.IYearGroupCoefficientService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.service.IYearOperatingCoefficientService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 年度经营业绩系数 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-19
 */
@Service
public class YearGroupCoefficientServiceImpl extends ServiceImpl<YearGroupCoefficientMapper, YearGroupCoefficient> implements IYearGroupCoefficientService {



	@Autowired
	private IYearOperatingCoefficientService yearOperatingCoefficientService;



	@Resource
	private YearOperatingCoefficientMapper yearOperatingCoefficientMapper;

	@Override
	public IPage<YearGroupCoefficient> list(Integer pageNum, Integer pageSize, String id, String coefficientCode, String year, String createOrgName, String createBy) {

		QueryWrapper<YearGroupCoefficient> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<YearGroupCoefficient> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(YearGroupCoefficient::getId, id);
		}

		if (StringUtils.isNotEmpty(coefficientCode)) {
			qw.lambda().like(YearGroupCoefficient::getCoefficientCode, coefficientCode);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(YearGroupCoefficient::getYear, year);
		}
		if (StringUtils.isNotEmpty(createOrgName)) {
			qw.lambda().like(YearGroupCoefficient::getCreateOrgName, createOrgName);
		}
		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(YearGroupCoefficient::getCreateBy, createBy);
		}

		IPage<YearGroupCoefficient> list = baseMapper.selectPage(page, qw);

		return list;
	}

	@Override
	public List<YearGroupCoefficient> listAll(String id, String coefficientCode, String year, String createOrgName, String createBy) {

		QueryWrapper<YearGroupCoefficient> qw = new QueryWrapper<>();


		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(YearGroupCoefficient::getId, id);
		}

		if (StringUtils.isNotEmpty(coefficientCode)) {
			qw.lambda().like(YearGroupCoefficient::getCoefficientCode, coefficientCode);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(YearGroupCoefficient::getYear, year);
		}

		if (StringUtils.isNotEmpty(createOrgName)) {
			qw.lambda().like(YearGroupCoefficient::getCreateOrgName, createOrgName);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(YearGroupCoefficient::getCreateBy, createBy);
		}

		List<YearGroupCoefficient> list = baseMapper.selectList(qw);

		return list;
	}

	@Transactional
	@Override
	public boolean saveGroup(YearGroupCoefficientDTO coefficientDTO) {
		QueryWrapper<YearGroupCoefficient> qw = new QueryWrapper<>();

		if (null != coefficientDTO && (null == coefficientDTO.getId() || "".equals(coefficientDTO.getId().trim()))) {
			if (StringUtils.isNotEmpty(coefficientDTO.getYear())) {
				qw.lambda().eq(YearGroupCoefficient::getYear, coefficientDTO.getYear());
			}
			StringBuilder sb = new StringBuilder();
			sb.append("ndxs").append("_");
			sb.append(coefficientDTO.getYear()).append("_").append("0001");
			coefficientDTO.setCoefficientCode(sb.toString());
			List<YearGroupCoefficient> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("已存在该年度系数");
			}
			coefficientDTO.setCreateTime(new Date());
		}else {
			coefficientDTO.setUpdateTime(new Date());
		}
		boolean b = super.saveOrUpdate(coefficientDTO);
		if(!b){
			throw new CustomException("年度系数保存失败");
		}

//		if(null == coefficientDTO.getCoefficientList() || coefficientDTO.getCoefficientList().size()<=0){
//			throw new CustomException("年度系数明细为空");
//		}
		for (YearOperatingCoefficient coefficient : coefficientDTO.getCoefficientList()){
			coefficient.setYearGroupCoefficientId(coefficientDTO.getId());
			coefficient.setYear(coefficientDTO.getYear());
			b =	yearOperatingCoefficientService.saveOrUpdate(coefficient);
			if(!b){
				throw new CustomException("年度系数保明细存失败");
			}
		}

		return b;
	}


	@Override
	public YearGroupCoefficientDTO getById(String id) {
		YearGroupCoefficientDTO byid = baseMapper.selectById(id);
		QueryWrapper<YearOperatingCoefficient> qw = new QueryWrapper<>();
		qw.lambda().eq(YearOperatingCoefficient::getYearGroupCoefficientId, id);
		List<YearOperatingCoefficient> coefficientList = yearOperatingCoefficientMapper.selectList(qw);
		byid.setCoefficientList(coefficientList);
		return byid;
	}


	@Transactional
	@Override
	public boolean saveOrUpdate(YearGroupCoefficient entity) {

		QueryWrapper<YearGroupCoefficient> qw = new QueryWrapper<>();

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			if (StringUtils.isNotEmpty(entity.getYear())) {
				qw.lambda().eq(YearGroupCoefficient::getYear, entity.getYear());
			}
			List<YearGroupCoefficient> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("已存在该年度系数");
			}
			entity.setCreateTime(new Date());
		}else {
			entity.setUpdateTime(new Date());
		}
		return super.saveOrUpdate(entity);
	}




	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {
		return super.removeByIds(idList);
	}
}
