package com.ims.eval.util;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;

public class ExcelUtil {



    /**
     * 读取Excel的内容
     *
     * @param fileType 文件类型，xls或xlsx
     * @param startRows 开始读取行数，比喻行头不需要读入 忽略的行数为1
     * @param ignoreRowBlank 是否忽略空行
     * @param is 文件输入流
     * @return 读出的Excel中数据的内容
     * @throws IOException duxxxxx
     */
    public static List<String[]> readData(String fileType, int startRows, boolean ignoreRowBlank, InputStream is) throws IOException {
        List<String[]> result = new ArrayList<>();

        Workbook wb = readExcel(fileType, is);
        for (int sheetIndex = 0; sheetIndex < wb.getNumberOfSheets(); sheetIndex++) {
            Sheet sheet = wb.getSheetAt(sheetIndex);

            for (int rowIndex = startRows, z = sheet.getLastRowNum(); rowIndex <= z; rowIndex++) {
                Row row = sheet.getRow(rowIndex);
                if (row == null) {
                    continue;
                }

                int rowSize = sheet.getRow(startRows).getLastCellNum();
                String[] values = new String[rowSize];
                boolean hasValue = false;
                for (int columnIndex = 0; columnIndex < rowSize; columnIndex++) {
                    String value = "";
                    Cell cell = row.getCell(columnIndex);
                    if (cell != null) {
                        // 注意：一定要设成这个，否则可能会出现乱码,后面版本默认设置
                        switch (cell.getCellType()) {
                            case HSSFCell.CELL_TYPE_STRING:
                                value = cell.getStringCellValue();
                                break;
                            case HSSFCell.CELL_TYPE_NUMERIC:
                                if (HSSFDateUtil.isCellDateFormatted(cell)) {
                                    Date date = cell.getDateCellValue();
                                    if (date != null) {
                                        value = new SimpleDateFormat("yyyy-MM-dd")
                                                .format(date);
                                    } else {
                                        value = "";
                                    }
                                } else {
                                    //value = new DecimalFormat("0").format(cell.getNumericCellValue());
                                    if (HSSFDateUtil.isCellDateFormatted(cell)) {
                                        value = String.valueOf(cell.getDateCellValue());
                                    } else {
                                        cell.setCellType(Cell.CELL_TYPE_STRING);
                                        String temp = cell.getStringCellValue();
                                        // 判断是否包含小数点，如果不含小数点，则以字符串读取，如果含小数点，则转换为Double类型的字符串
                                        if (temp.indexOf(".") > -1) {
                                            value = String.valueOf(new Double(temp)).trim();
                                        } else {
                                            value = temp.trim();
                                        }
                                    }
                                }
                                break;
                            case HSSFCell.CELL_TYPE_FORMULA:
                                // 导入时如果为公式生成的数据则无值
                                if (!cell.getStringCellValue().equals("")) {
                                    value = cell.getStringCellValue();
                                } else {
                                    value = cell.getNumericCellValue() + "";
                                }
                                break;
                            case HSSFCell.CELL_TYPE_BLANK:
                                break;
                            case HSSFCell.CELL_TYPE_ERROR:
                                value = "";
                                break;
                            case HSSFCell.CELL_TYPE_BOOLEAN:
                                value = (cell.getBooleanCellValue() == true ? "Y"

                                        : "N");
                                break;
                            default:
                                value = "";
                        }
                    }
                    values[columnIndex] = value;
                    if (!value.isEmpty()) {
                        hasValue = true;
                    }
                }
                if (!ignoreRowBlank || hasValue) {//不为忽略空行模式或不为空行
                    result.add(values);
                }
            }
        }
        return result;
    }

    //读取excel
    public static Workbook readExcel(String fileType, InputStream is) throws IOException {
        if ("xls".equals(fileType)) {
            return new HSSFWorkbook(is);
        } else if ("xlsx".equals(fileType)) {
            return new XSSFWorkbook(is);
        } else {
            throw new IllegalArgumentException("不支持的文件类型，仅支持xls和xlsx");
        }
    }

    /**
     * 去掉字符串右边的空格
     *
     * @param str 要处理的字符串
     * @return 处理后的字符串
     */
    private static String rightTrim(String str) {
        if (str == null) {
            return "";
        }
        int length = str.length();
        for (int i = length - 1; i >= 0; i--) {
            if (str.charAt(i) != 0x20) {
                break;
            }
            length--;
        }
        return str.substring(0, length);
    }

	public static byte[] export(String sheetTitle, String[] title, List<Object> list) {

		HSSFWorkbook wb = new HSSFWorkbook();//创建excel表
		HSSFSheet sheet = wb.createSheet(sheetTitle);
		sheet.setDefaultColumnWidth(20);//设置默认行宽

		//表头样式（加粗，水平居中，垂直居中）
		HSSFCellStyle cellStyle = wb.createCellStyle();
		cellStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);//水平居中
		cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);//垂直居中
		//设置边框样式
		cellStyle.setBorderBottom(HSSFCellStyle.BORDER_THIN); //下边框
		cellStyle.setBorderLeft(HSSFCellStyle.BORDER_THIN);//左边框
		cellStyle.setBorderTop(HSSFCellStyle.BORDER_THIN);//上边框
		cellStyle.setBorderRight(HSSFCellStyle.BORDER_THIN);//右边框

		HSSFFont fontStyle = wb.createFont();
		fontStyle.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);

		cellStyle.setFont(fontStyle);

		//标题样式（加粗，垂直居中）
		HSSFCellStyle cellStyle2 = wb.createCellStyle();
		cellStyle2.setVerticalAlignment(VerticalAlignment.CENTER);//垂直居中
		cellStyle2.setFont(fontStyle);

		//设置边框样式
		cellStyle2.setBorderBottom(HSSFCellStyle.BORDER_THIN); //下边框
		cellStyle2.setBorderLeft(HSSFCellStyle.BORDER_THIN);//左边框
		cellStyle2.setBorderTop(HSSFCellStyle.BORDER_THIN);//上边框
		cellStyle2.setBorderRight(HSSFCellStyle.BORDER_THIN);//右边框

		//字段样式（垂直居中）
		HSSFCellStyle cellStyle3 = wb.createCellStyle();
		cellStyle3.setVerticalAlignment(VerticalAlignment.CENTER);//垂直居中

		//设置边框样式
		cellStyle3.setBorderBottom(HSSFCellStyle.BORDER_THIN); //下边框
		cellStyle3.setBorderLeft(HSSFCellStyle.BORDER_THIN);//左边框
		cellStyle3.setBorderTop(HSSFCellStyle.BORDER_THIN);//上边框
		cellStyle3.setBorderRight(HSSFCellStyle.BORDER_THIN);//右边框

		//创建表头
		HSSFRow row = sheet.createRow(0);
		row.setHeightInPoints(20);//行高

		HSSFCell cell = row.createCell(0);
		cell.setCellValue(sheetTitle);
		cell.setCellStyle(cellStyle);

		sheet.addMergedRegion(new CellRangeAddress(0,0,0,(title.length-1)));

		//创建标题
		HSSFRow rowTitle = sheet.createRow(1);
		rowTitle.setHeightInPoints(20);

		HSSFCell hc;
		for (int i = 0; i < title.length; i++) {
			hc = rowTitle.createCell(i);
			hc.setCellValue(title[i]);
			hc.setCellStyle(cellStyle2);
		}

		byte result[] = null;

		ByteArrayOutputStream out = null;

		try {
			//创建表格数据
			Field[] fields;
			int i = 2;

			for (Object obj : list) {
				fields = obj.getClass().getDeclaredFields();

				HSSFRow rowBody = sheet.createRow(i);
				rowBody.setHeightInPoints(20);

				int j = 0;
				for (Field f : fields) {

					f.setAccessible(true);

					Object va = f.get(obj);
					if (null == va) {
						va = "";
					}

					hc = rowBody.createCell(j);
					hc.setCellValue(va.toString());
					hc.setCellStyle(cellStyle3);

					j++;
				}

				i++;
			}

			out = new ByteArrayOutputStream();
			wb.write(out);
			result =  out.toByteArray();
		} catch (Exception ex) {
			Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, ex);
		} finally{
			try {
				if(null != out){
					out.close();
				}
			} catch (IOException ex) {
				Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, ex);
			} finally{
				try {
					wb.close();
				} catch (IOException ex) {
					Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		}

		return result;
	}
}
