package com.ims.eval.util;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.Version;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.Map;

/**
 * @author hlf
 * @date 2023/5/10 14:52
 * 文件说明：导出word文档工具类
 */
public class WordUtil {

	/**
	 * 生成 word 文档方法
	 *
	 * @param dataMap      要填充的数据
	 * @param templateName 模版名称
	 * @param fileName     要输出的文件路径
	 * @throws Exception 抛出的异常
	 */
	public static void generateWord(Map<String, Object> dataMap, String templateName, String path, String fileName) throws Exception {
		// 设置FreeMarker的版本和编码格式
		Configuration configuration = new Configuration(new Version("2.3.28"));
		configuration.setDefaultEncoding("UTF-8");
		configuration.setClassicCompatible(true);
		// 设置FreeMarker生成Word文档所需要的模板的路径
		// configuration.setDirectoryForTemplateLoading(new File("D:/home/templates"));
		// 此处把模版文件都放在 resources 下的 templates 中
		configuration.setClassForTemplateLoading(WordUtil.class, "/templates");
		// 设置FreeMarker生成Word文档所需要的模板
		Template tem = configuration.getTemplate(templateName, "UTF-8");
		// 查看路径下是否有存放文档的文件夹 没有就创建
		File file = new File(path);
		if (!file.exists()){
			file.mkdirs();
		}
		// 创建一个Word文档的输出流
		Writer out = new BufferedWriter(new OutputStreamWriter(Files.newOutputStream(new File(path + fileName).toPath()), StandardCharsets.UTF_8));
		// FreeMarker使用Word模板和数据生成Word文档
		tem.process(dataMap, out);
		out.flush();
		out.close();
	}
}
